#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Install a package and then make available a new version of the same package.
# This is similar to test 3 but we put a file "test1" in version 1.0 of the
# package and a file "test2" in version 2.0 of the package. Check that on
# upgrade, "test1" is removed and "test2" is created.
#

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()

f = open("test1", "w")
f.write("Test")
f.close()
pkg = opk.Opk(Package="a", Version="1.0")
pkg.write(data_files=["test1"])
o.addOpk(pkg)
o.write_list()
os.unlink("test1")

opkgcl.update()

opkgcl.install("a")
if not opkgcl.is_installed("a"):
    opk.fail("Package 'a' installed but reports as not installed.")
if not os.path.exists("%s/test1" % cfg.offline_root):
    opk.fail("Package 'a' installed but file 'test1' not created.")

# Make a new version of 'a' available
f = open("test2", "w")
f.write("Test")
f.close()
pkg = opk.Opk(Package="a", Version="2.0")
pkg.write(data_files=["test2"])
o.addOpk(pkg)
o.write_list()
os.unlink("test2")

opkgcl.update()
opkgcl.upgrade()

if not opkgcl.is_installed("a", "2.0"):
    opk.fail("New version of package 'a' available during upgrade but was not installed")
if opkgcl.is_installed("a", "1.0"):
    opk.fail("Package 'a' upgraded but old version still installed.")

if not os.path.exists("%s/test2" % cfg.offline_root):
    opk.fail("Package 'a' upgraded but new file 'test2' not created.")
if os.path.exists("%s/test1" % cfg.offline_root):
    opk.fail("Package 'a' upgraded but old file 'test1' not removed.")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Report and fix from "paradox.kahn" in a google code comment to r543.
#
# What steps will reproduce the problem?
# ======================================
#
# 1. Create a package containing a symlink to a file with a long file name
#     (longer than 100 characters)
# 2. Install the package
#
#
# What is the expected output? What do you see instead?
# =====================================================
#
# The symlink should be created and point to the appropriate file. Instead, the
# symlink is not created at all.
#
#
# Status
# ======
#
# Fixed in r544.

import os
import opk, cfg, opkgcl

opk.regress_init()

long_filename = 110*"a"

os.symlink(long_filename, "linky")
a = opk.Opk(Package="a")
a.write(data_files=["linky"])
os.unlink("linky")
opkgcl.install("a_1.0_all.opk")

if not opkgcl.is_installed("a"):
    opk.fail("Package 'a' not installed.")

if not os.path.lexists("{}/linky".format(cfg.offline_root)):
    opk.fail("symlink to file with a name longer than 100 "
             "characters not created.")

opkgcl.remove("a")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Reporter: Kay Hackmack
#
# Due to an error on our side our feed's Packages/Packages.gz was generated
# like this (xyz is some package-specific data):
#
# Package: xyz
# Version: 20.7.0.79-0+d79
# Architecture: core2-64
# Maintainer: Example User <user@example.com>
# Source: xyz
# Section: Add-Ons
# Priority: extra
# Depends: xyz
# Homepage: https://urldefense.com/v3/__https://www.example.com/__;!!FbZ0ZwI3Qg!rQIEh4JM-KkE8EhjDYoIPcyuh3a_FnCKbMxd_rUY8l2hFZ6-vHs0sRM4r_iSUU58fYHXgj-I54BpJXMGFUl2vQ$
# DisplayVersion: 20.7
# DisplayName: xyz
# UserVisible:  yes
# Description: xyz
# xyz
# MD5Sum:
# Size:
# Filename: xyz.ipk
#
# After opkg on the target accessed/cached that feed it starts segfaulting
# when running any command.
# The key here seems to be that MD5Sum and/or Size: are empty. I'm not sure
# which entry (MD5Sum or Size) is causing the crash or both at the moment.

import os
import opk, cfg, opkgcl

BOGUS_SIZES = [
    "",                      # empty string
    "-1",                    # negative number
    "-9223372036854775808",  # LLONG_MIN - 1
    "18446744073709551616",  # ULLONG_MAX + 1
    "foo",                   # no digits
    "42fortytwo",            # digits but also (non-hex) digits
]

for bogosity in BOGUS_SIZES:
    opk.regress_init()

    o = opk.OpkGroup()
    o.add(Package="xyz", Version="1.0")
    o.write_opk()

    # write out a package list with bogus size
    with open('Packages', 'w') as f:
        for pkg in o.opk_list:
            for k in pkg.control.keys():
                f.write('{}: {}\n'.format(k, pkg.control[k]))
            fname = pkg.control['Filename']
            md5sum = opk.md5sum_file(fname)
            f.write('Size: {}\n'.format(bogosity))
            f.write('MD5Sum: {}\n'.format(md5sum))
            f.write('\n')

    if opkgcl.update() != 0:
        fail("failed to update package list")

    (status, output) = opkgcl.opkgcl('list')
    if status != 0:
        # Even with the ERROR printed this is still a success, in that
        # it should not be returning code 139 (SIGSEGV)
        opk.fail("opkg list returned nonzero exit code {}".format(status))
    if "Failed to parse Size line for xyz" not in output:
        opk.fail("expected to find error in output, but did not")

    (status, output) = opkgcl.opkgcl('info xyz')
    if status != 0:
        opk.fail("opkg list returned nonzero exit code {}".format(status))
    if "Failed to parse Size line for xyz" not in output:
        opk.fail("expected to find error in output, but did not")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Reporter: paul.spangler@ni.com
#
# What steps will reproduce the problem?
# ======================================
#
# 1.- Create package 'a' which installs a file in
#     in an existing directory. At package creation
#     time set the dir permissions to 0o777
# 2.- Create a rootfs that has the directory where
#     package 'a' will install a file. Set the permisions
#     on that directory to 0o755.
# 3.- Install 'a'
#
# What is the expected output? What do you see instead?
# ======================================
#
# The directory permisions change from 0o755 to 0o777.
# It should stay as 0o755 (dpkg behavior)
#

import os
import opk, cfg, opkgcl

opk.regress_init()

subdir = "dir/subdir"
filename = subdir + "/myfile"
rootfs_subdir = "{}/{}".format(cfg.offline_root, subdir)

os.makedirs(subdir, 0o755)
os.chmod(subdir, 0o777)
open(filename, "w").close()
a = opk.Opk(Package="a")
a.write(data_files=[subdir, filename])
os.unlink(filename)
os.rmdir(subdir)

os.makedirs(rootfs_subdir, 0o755)
opkgcl.install("a_1.0_all.opk")

if not opkgcl.is_installed("a"):
     opk.fail("Package 'a' installed but reports as not installed.")

if os.stat(rootfs_subdir).st_mode != 0o40755:
     opk.fail("Install operation incorrectly changed existing directory permissions")

opkgcl.remove("a")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Reported by KirGeNe, Feb 6, 2014
# Test case modified by Paul Barker
#
# What steps will reproduce the problem?
# 1. Create package 'a' in repo, version 1.0, provides 'b'.
# 2. Create package 'x' in repo, depends on 'a'.
# 3. Create package 'y' in repo, depends on 'b'.
# 4. Install 'x' & 'y'.
# 5. Change 'a' in repo to version 2.0, no longer provides 'b'.
# 6. Create package 'b' in repo.
# 7. Do upgrade.
#
# What is the expected output?
# Packages 'a' v2.0 and 'b' v1.0 are installed.
#
# What do you see instead?
# Only package 'a' v2.0 is installed. Thus provider of 'b' is missing.
#

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a", Version="1.0", Provides="b")
o.add(Package="x", Depends="a")
o.add(Package="y", Depends="b")
o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.install("x")
if not opkgcl.is_installed("x"):
    opk.fail("Package 'x' failed to install.")

opkgcl.install("y")
if not opkgcl.is_installed("y"):
    opk.fail("Package 'y' failed to install.")

if not opkgcl.is_installed("a"):
    opk.fail("Package 'a' not installed despite being a dependency of installed packages.")

o = opk.OpkGroup()
o.add(Package="a", Version="2.0", Depends="b")
o.add(Package="b")
o.add(Package="x", Depends="a")
o.add(Package="y", Depends="b")
o.write_opk()
o.write_list()

opkgcl.update()
opkgcl.upgrade()

if not opkgcl.is_installed("a", "2.0"):
    opk.fail("New version of package 'a' available during upgrade but was not installed")
if opkgcl.is_installed("a", "1.0"):
    opk.fail("Package 'a' upgraded but old version still installed.")

if not opkgcl.is_installed("b"):
    opk.fail("Package 'b' not installed despite being a dependency of installed packages.")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Verifies intercept scripts can be disabled by an opkg conf token.
#
# Installs a package with preinst, postinst, prerm, and postrm scripts
# calling `ls` in sysroot with an ls intercept. The intercepted ls
# script logs a message to indicate when it's called. The test sequence
# runs with intercepts_dir unset and intercepts_dir=/dev/null to verify
# ls is only intercepted in the former case and not the latter.
#

import os, tempfile, errno
import opk, cfg, opkgcl

opk.regress_init()

def readFile(path):
    with open(path, 'r') as f:
        return f.read()

def truncFile(path):
    with open(path, 'w') as f:
        pass

def appendFile(path, string):
    with open(path, 'a') as f:
        f.write(string)

def touch_dir(path):
    try:
        os.makedirs(path)
    except OSError as e:
        if e.errno != errno.EEXIST:
            raise

# Log file appended to by the intercept script.
# Test sequence (below) removes this file before installing packages,
# then compares it's contents against an expected value.
TEST_LOG = os.path.join(cfg.offline_root, "intercepts_test.log")

def prepare_sysroot():
    '''
    Creates intercept script and test package.
    '''
    testprefix=cfg.offline_root+os.environ['DATADIR']
    # Add a intercept script for ls to the sysroot
    touch_dir('%s/opkg/intercept' % testprefix)
    with open('%s/opkg/intercept/ls' % testprefix, 'w') as f:
        os.fchmod(f.fileno(), 0o755)
        f.write('\n'.join([
            '#!/bin/sh',
            'set -e',
            'exeFileName="`basename "$1"`"',
            'echo -n \"intercept from $exeFileName:\" >> \'%s\'' % TEST_LOG,
            'echo "ls $* was intercepted"',
        ]))

    o = opk.OpkGroup()

    # Add a test package
    a = opk.Opk(Package='a')
    a.postinst = '\n'.join([
        '#!/bin/sh',
        'set -e',
        'ls "$0"',
    ])
    o.addOpk(a)

    o.write_opk()
    o.write_list()

prepare_sysroot()
opkgcl.update()

# First install with intercepts enabled
truncFile(TEST_LOG)
if opkgcl.install('a') != 0:
    opk.fail('Failed to install test package')
if opkgcl.remove('a') != 0:
    opk.fail('Failed to remove test package')
if readFile(TEST_LOG) != 'intercept from a.postinst:':
    opk.fail('Unexpected intercept log')

sysconfdir=os.environ['SYSCONFDIR']
testconfdir=cfg.offline_root+sysconfdir
# Reconfigure opkg to disable intercepts
appendFile('%s/opkg/opkg.conf' % testconfdir, 'option intercepts_dir /dev/null\n')

# Re-run the test
truncFile(TEST_LOG)
if opkgcl.install('a') != 0:
    opk.fail('Failed to install test package')
if opkgcl.remove('a') != 0:
    opk.fail('Failed to remove test package')
if readFile(TEST_LOG) != '':
    opk.fail('Unexpected intercept log')



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Test opkg when specifying a constrained/fuzzy version to install via:
#       opkg install pkg_name<=pkg_version
#
# 1 - Create packages 'a(1.0)', 'a(2.0)' and 'a(3.0)'
# 2 - Install 'a<=2.0'      # Should install 2.0
# 3 - Upgrade               # Should install 3.0
# 4 - Install 'a<<2.0', with --force-downgrade  # Should install 1.0
# 5 - Install 'a>=2.0'      # Should install 3.0
# 6 - Install 'a>=4.0'      # Should fail
#

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a", Version ="1.0")
o.add(Package="a", Version ="2.0")
o.add(Package="a", Version ="3.0")
o.write_opk()
o.write_list()

opkgcl.update()

# NOTE: Install commands need to be quoted here, since we call subprocess
# with shell=True (meaning characters like '<' would get handled by the shell)
opkgcl.install("'a<=2.0'")
if not opkgcl.is_installed("a", "2.0"):
    opk.fail("Package 'a', Version '2.0', failed to install.")

opkgcl.upgrade("a")
if not opkgcl.is_installed("a", "3.0"):
    opk.fail("Package 'a' was not upgraded to Version '3.0'.")

opkgcl.install("'a<<2.0'", "--force-downgrade")
if not opkgcl.is_installed("a", "1.0"):
    opk.fail("Package 'a' was not downgraded to Version '1.0'.")

opkgcl.install("'a>=2.0'")
if not opkgcl.is_installed("a", "3.0"):
    opk.fail("Package 'a', Version '3.0', failed to install.")

opkgcl.remove("a")
opkgcl.install("'a>=4.0'")
if opkgcl.is_installed("a"):
    opk.fail("Package 'a' was installed incorrectly.")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Reporter: muusclaus
#
# What steps will reproduce the problem?
########################################
#
#  1.- Create package 'y' (1.0), which Depends on 'x'
#  2.- Create package 'x' (1.0)
#  3.- Update
#  4.- Remove package x
#  5.- Install 'y'. The operation will fail, but will install
#      'y'
#  6.- Create package 'x' (2.0)
#  7.- Install 'x'
#
# What is the expected output? What do you see instead?
########################################
#
#  Package 'x' (2.0) should be installed
#  Instead, opkg tries to install 'x' (1.0), and fails.
#

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
x = o.add(Package="x", Version="1.0")
o.add(Package="y", Version="1.0", Depends="x")

o.write_opk()
o.write_list()

opkgcl.update()

# Will force failure to install 'x', even though it's listed in Packages
os.unlink("x_1.0_all.opk")

opkgcl.install("y")

o.removeOpk(x)
o.add(Package="x", Version="2.0")
o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.install("x")
if not opkgcl.is_installed("x", version="2.0"):
    opk.fail("Failed to install 'x' (2.0)")

opkgcl.remove("y")
if opkgcl.is_installed("y"):
    opk.fail("Failed to uninstall 'y'")

opkgcl.remove("x")
if opkgcl.is_installed("x"):
    opk.fail("Failed to uninstall 'x'")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Install a package 'x' which PROVIDES and CONFLICTS 'v', indicating that 'v' is
# a virtual package and no other provider of this virtual package should be
# installed whilst 'x' is installed. Then try to install 'y' which also PROVIDES
# 'v'.

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="x", Provides="v", Conflicts="v")
o.add(Package="y", Provides="v")
o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.install("x")
if not opkgcl.is_installed("x"):
    opk.fail("Package 'x' installed but reports as not installed.")

# Now try to install "y", which should fail

opkgcl.install("y")
if opkgcl.is_installed("y"):
    opk.fail("Package 'y' installed despite conflict with 'v' provided by 'x'.")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Install a package 'a' which depends on a second package 'b'. Check that both
# are installed, that package 'a' does not report as automatically installed but
# that package 'b' does report as automatically installed.
#

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a", Depends="b")
o.add(Package="b")
o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.install("a")
if not opkgcl.is_installed("a"):
    opk.fail("Package 'a' installed but does not report as installed.")
if not opkgcl.is_installed("b"):
    opk.fail("Package 'b' should be installed as a dependency of 'a' but does not report as installed.")

# Check the packages are marked correctly
if opkgcl.is_autoinstalled("a"):
    opk.fail("Package 'a' explicitly installed by reports as auto installed.")
if not opkgcl.is_autoinstalled("b"):
    opk.fail("Package 'b' installed as a dependency but does not report as auto installed.")

# Check that trying to remove 'b' fails and doesn't change the system (issue 9862)
if not opkgcl.remove("b"):
        opk.fail("Package 'b' should not be allowed to be uninstalled")
if not opkgcl.is_installed("a"):
        opk.fail("Package 'a' was incorrectly uninstalled")
if not opkgcl.is_installed("b"):
        opk.fail("Package 'b' was incorrectly uninstalled")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Test opkg when specifying a virtual package to install:
#       opkg install <virtual_pkg_name>
#

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a", Version='1.0', Provides="virt")
o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.install("virt")
if not opkgcl.is_installed("a", "1.0"):
    opk.fail("Package 'virt' (provided by 'a') failed to install.")

o.add(Package="a", Version='2.0', Provides="virt")
o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.upgrade("virt")
if not opkgcl.is_installed("a", "2.0"):
    opk.fail("Package 'virt' (provided by 'a') failed to upgrade.")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Create package 'x(1.0)' with architecture a and 'x(2.0)' with architecture b.
# Install 'x' with --add-arch a:2 and --add-arch b:1.
# Check that 'x(2.0)' is installed. Then remove 'x'.
# Install 'x' with --add-arch a:2 and --add-arch b:1 and --prefer-arch-to-version.
# Check that 'x(1.0)' is installed.
#

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="x", Version="1.0", Architecture="a")
o.add(Package="x", Version="2.0", Architecture="b")

o.write_opk()
o.write_list()

opkgcl.update()

arch_flags = "--add-arch a:2 --add-arch b:1"

# install should prioritize version
opkgcl.install("x", arch_flags)
if opkgcl.is_installed("x", "1.0", arch_flags):
        opk.fail("Package 'x(2.0)' available but 1.0 installed")
if not opkgcl.is_installed("x", "2.0", arch_flags):
        opk.fail("Package 'x(2.0)' available but was not installed.")

opkgcl.remove("x", arch_flags)
if opkgcl.is_installed("x", "2.0", arch_flags):
        opk.fail("Package 'x' removed but reports as installed.")

# prefer-arch-to-version should prefer the architecture
opkgcl.install("x", arch_flags + " --prefer-arch-to-version")
if opkgcl.is_installed("x", "2.0", arch_flags):
    opk.fail("Package 'x(1.0) has preferred arch but 2.0 was installed")
if not opkgcl.is_installed("x", "1.0", arch_flags):
    opk.fail("Package 'x(1.0)' with preferred arch available but was not installed.")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Reporter: markus.lehtonen@linux.intel.com
#
# What steps will reproduce the problem?
########################################
#
# 1. Create package 'a', which DEPENDS on 'b'.
# 2. Create package 'b', which DEPENDS on 'd'.
# 3. Create package 'c', which PROVIDES 'd' and CONFLICTS with 'e''.
# 4. Create package 'e', which PROVIDES 'd' and CONFLICTS with 'c'.
# 5. Install 'a'
#
# What is the expected output? What do you see instead?
########################################
# Packages 'a', 'b' and 'c' or 'e' should be installed.
#
# Instead, opkg errors out:
#    Collected errors:
#     * check_conflicts_for: The following packages conflict with c:
#     * check_conflicts_for:         e*
#

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a", Depends="b")
o.add(Package="b", Depends="d")
o.add(Package="c", Provides="d", Conflicts="e")
o.add(Package="e", Provides="d", Conflicts="c")
o.write_opk()
o.write_list()

opkgcl.update()

status, stdout = opkgcl.opkgcl(" --force-postinstall install a")
if status != 0:
    opk.fail("Return value was different than 0")
if "Collected errors" in stdout:
    opk.fail("Command returned an error")
if not opkgcl.is_installed("a"):
    opk.fail("Package 'a' installed but does not report as installed.")
if not opkgcl.is_installed("b"):
    opk.fail("Package 'b' should be installed as a dependency of 'a' but does not report as installed.")
if not opkgcl.is_installed("c") and not opkgcl.is_installed("e"):
    opk.fail("Package 'c' or 'e' should be installed as a dependency of 'b' but does not report as installed.")
if opkgcl.is_installed("c") and opkgcl.is_installed("e"):
    opk.fail("Package 'c' or 'e' should be installed as a dependency of 'b' but not both.")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Reporter: Paul Barker
#
# What steps will reproduce the problem?
# ======================================
#
# 1. Create packages A and C which both recommend B.
# 2. Create package B.
# 3. Install A and C.
# 4. Remove A with --autoremove
#
#
# What is the expected output? What do you see instead?
# =====================================================
#
# Expect B to remain installed as it is still depended on by C. Instead, B is
# removed.
#
# This test case was initially part of issue58.py but the failure was not as
# described in issue 58 in the bug tracker, so this was split out.
#

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a", Recommends="b")
o.add(Package="b")
o.add(Package="c", Recommends="b")
o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.install("a")
opkgcl.install("c")

opkgcl.remove("a", "--autoremove")
if not opkgcl.is_installed("b"):
    opk.fail("Pacakge 'b' removed despite remaining recommending package 'c'.")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Issue 124, reported by muusclaus
#
# This is an additional test case for issue 124 to cover the case where multiple
# packages depend on a common package with version constraints which change
# during an upgrade. The initial bugfix correctly handled the case with a single
# dependent but with multiple dependent packages it became impossible to upgrade
# any of them. This test case has been added to ensure that the final bugfix
# allows all the packages to be upgraded together.

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a", Version="1.0", Depends="b (= 1.0)")
o.add(Package="a_second", Version="1.0", Depends="b (=1.0)")
o.add(Package="b", Version="1.0")
o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.install("a a_second")
if not opkgcl.is_installed("a"):
    opk.fail("Package 'a' failed to install.")
if not opkgcl.is_installed("a_second"):
    opk.fail("Package 'a_second' failed to install.")
if not opkgcl.is_installed("b"):
    opk.fail("Package 'b' not installed despite dependency from package 'a'.")

o = opk.OpkGroup()
o.add(Package="a", Version="1.1", Depends="b (= 1.1)")
o.add(Package="a_second", Version="1.1", Depends="b (= 1.1)")
o.add(Package="b", Version="1.1")
o.write_opk()
o.write_list()

opkgcl.update()

# 'opkg upgrade b' should fail as it won't upgrade a or a_second
opkgcl.upgrade("b")

# Check 'a' has not been upgraded
if opkgcl.is_installed("a", "1.1"):
    opk.xfail("[libsolv] Package 'a' upgraded despite not being listed in packages to upgrade.")
if not opkgcl.is_installed("a", "1.0"):
    opk.fail("Package 'a' version 1.0 removed.")

# Check 'a_second' has not been upgraded
if opkgcl.is_installed("a_second", "1.1"):
    opk.fail("Package 'a_second' upgraded despite not being listed in packages to upgrade.")
if not opkgcl.is_installed("a_second", "1.0"):
    opk.fail("Package 'a_second' version 1.0 removed.")

# Check 'b' has not been upgraded
if opkgcl.is_installed("b", "1.1"):
    opk.fail("Package 'b' upgraded despite breaking dependency of package 'a'.")
if not opkgcl.is_installed("b", "1.0"):
    opk.fail("Package 'b' version 1.0 removed.")

# 'opkg upgrade a' should fail as it won't upgrade a_second
opkgcl.upgrade("a")

# Check 'a' has not been upgraded
if opkgcl.is_installed("a", "1.1"):
    opk.fail("Package 'a' upgraded despite not being listed in packages to upgrade.")
if not opkgcl.is_installed("a", "1.0"):
    opk.fail("Package 'a' version 1.0 removed.")

# Check 'a_second' has not been upgraded
if opkgcl.is_installed("a_second", "1.1"):
    opk.fail("Package 'a_second' upgraded despite not being listed in packages to upgrade.")
if not opkgcl.is_installed("a_second", "1.0"):
    opk.fail("Package 'a_second' version 1.0 removed.")

# Check 'b' has not been upgraded
if opkgcl.is_installed("b", "1.1"):
    opk.fail("Package 'b' upgraded despite breaking dependency of package 'a'.")
if not opkgcl.is_installed("b", "1.0"):
    opk.fail("Package 'b' version 1.0 removed.")

# 'opkg upgrade --combine a a_second' should succeed and upgrade a, a_second and b
opkgcl.upgrade("--combine a a_second")

# Check 'a' has been upgraded
if not opkgcl.is_installed("a", "1.1"):
    opk.fail("Package 'a' failed to upgrade.")
if opkgcl.is_installed("a", "1.0"):
    opk.fail("Package 'a' version 1.0 not removed despite successful upgrade.")

# Check 'a_second' has been upgraded
if not opkgcl.is_installed("a_second", "1.1"):
    opk.fail("Package 'a_second' failed to upgrade.")
if opkgcl.is_installed("a_second", "1.0"):
    opk.fail("Package 'a_second' version 1.0 not removed despite successful upgrade.")

# Check 'b' has been upgraded
if not opkgcl.is_installed("b", "1.1"):
    opk.fail("Package 'b' failed to upgrade.")
if opkgcl.is_installed("b", "1.0"):
    opk.fail("Package 'b' version 1.0 not removed despite successful upgrade.")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Create package 'a' which depends on 'b'
# Install 'a' with --nodeps and check that 'b' is not installed
#

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a", Depends="b")
o.add(Package="b")

o.write_opk()
o.write_list()

opkgcl.update()

# installing with --nodeps should not install deps
opkgcl.install("a", "--nodeps")
if not opkgcl.is_installed("a"):
        opk.fail("Package 'a' installed but reports as not installed")
if opkgcl.is_installed("b"):
        opk.fail("Package 'a' installed with --nodeps but dependency 'b' installed.")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Reporter: Eric Yu
#
# What steps will reproduce the problem?
# ======================================
#
# 1. Install a package 'x'
# 2. Install a package 'a' which depends on 'b' and 'c', where 'c' conflicts with 'x'
#
# What is the expected output? What do you see instead?
# =====================================================
#
# Expected:
# 'x' remains installed, and 'a', 'b', and 'c' remain uninstalled because
# 'c' conflicts with 'x'
#
# Actual:
# 'x' remains installed. 'a' and 'c' are not installed, but 'b' is installed.
#
# Status
# ======
# Open
#

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="x")
o.add(Package="a", Depends="b, c")
o.add(Package="b")
o.add(Package="c", Conflicts="x");


o.write_opk()
o.write_list()
opkgcl.update()

opkgcl.install("x")
if not opkgcl.is_installed("x"):
    opk.fail("Package 'x' installed but reports as not installed.")

opkgcl.install("a")
if opkgcl.is_installed("c"):
    opk.fail("Package 'c' installed but conflicts with x.")
if opkgcl.is_installed("a"):
    opk.fail("Package 'a' installed but dependency 'c' is not installed due to a conflict.")
if opkgcl.is_installed("b"):
    opk.fail("Package 'b' installed and orphaned.")

opkgcl.remove("x")
opkgcl.remove("a")
opkgcl.remove("b")
opkgcl.remove("c")



#!/usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a", Version="1.12A")
o.add(Package="a", Version="1.12B")
o.add(Package="b", Version="1.2")
o.add(Package="b", Version="1.12")
o.add(Package="c", Version="001.1535A")
o.add(Package="c", Version="001.CIBUILDX20160919_153521")
o.add(Package="d", Version="1:1.12B")
o.add(Package="d", Version="2:1.12A")
o.add(Package="e", Version="2.0")
o.add(Package="e", Version="1:1.0")
o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.install("a")
if opkgcl.is_installed("a", "1.12A"):
    opk.fail("Wrong 'a' package version installed")
elif not opkgcl.is_installed("a", "1.12B"):
    opk.fail("No 'a' package installed")

opkgcl.install("b")
if opkgcl.is_installed("b", "1.2"):
    opk.fail("Wrong 'b' package version installed")
elif not opkgcl.is_installed("b", "1.12"):
    opk.fail("No 'b' package installed")

opkgcl.install("c")
if opkgcl.is_installed("c", "001.1535A"):
    opk.fail("Wrong 'c' package version installed")
elif not opkgcl.is_installed("c", "001.CIBUILDX20160919_153521"):
    opk.fail("No 'c' package installed")

opkgcl.install("d")
if opkgcl.is_installed("d", "1:1.12B"):
    opk.fail("Wrong 'd' package version installed")
elif not opkgcl.is_installed("d", "2:1.12A"):
    opk.fail("No 'd' package installed")

opkgcl.install("e")
if opkgcl.is_installed("e", "2.0"):
    opk.fail("Wrong 'e' package version installed")
elif not opkgcl.is_installed("e", "1:1.0"):
    opk.fail("No 'e' package installed")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Test opkg list-upgradable behavior.

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a", Version ="1.0")
o.add(Package="a", Version ="2.0")
o.add(Package="a", Version ="3.0")
o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.install("a=1.0")
if not opkgcl.is_installed("a", "1.0"):
    opk.fail("Package 'a', Version '1.0', failed to install.")

if opkgcl.is_upgradable("a", "2.0"):
    opk.fail("Package 'a', is upgradable to version '2.0', though a better candidate is available.")

if not opkgcl.is_upgradable("a", "3.0"):
    opk.fail("Package 'a', is not upgradable to version '3.0'.")

opkgcl.install("a=2.0")
if not opkgcl.is_installed("a", "2.0"):
    opk.fail("Package 'a', Version '2.0', failed to install.")

if not opkgcl.is_upgradable("a", "3.0"):
    opk.fail("Package 'a', is not upgradable to version '3.0'.")

opkgcl.install("a=3.0")
if not opkgcl.is_installed("a", "3.0"):
    opk.fail("Package 'a', Version '3.0', failed to install.")

if opkgcl.is_upgradable("a"):
    opk.fail("Package 'a', is upgradable, but is already the latest.")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Create packages 'a', 'b' which recommends 'a' and 'c' which depends on 'a'.
# Install 'b' with '--add-exclude a' and ensure 'b' is installed but 'a' is not.
# Install 'c' with '--add-exclude a' and ensure 'c' and 'a' are not installed.
#

import os
import re
import opk, cfg, opkgcl

opk.regress_init()

_, version = opkgcl.opkgcl("--version")
libsolv = True if "libsolv" in version else False

o = opk.OpkGroup()
o.add(Package="a1")
o.add(Package="b", Recommends="a1, a2")
o.add(Package="c", Depends="a1")
o.add(Package="a2")
o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.install("b", "--add-exclude a1")
if opkgcl.is_installed("a1"):
    opk.fail("Package 'a1' installed despite being excluded.")
if not opkgcl.is_installed("b"):
    opk.fail("Package 'b' not installed, excluded pacakge was only a recommendation.")
if not opkgcl.is_installed("a2"):
    opk.fail("Package 'a2' not installed, excluded pacakge was only a recommendation.")

opkgcl.remove("b a2", "--force-depends")

opkgcl.install("c", "--add-exclude a1")
if opkgcl.is_installed("a1"):
    opk.fail("Package 'a1' installed despite being excluded.")
if opkgcl.is_installed("c"):
    opk.fail("Package 'c' installed despite required package being excluded.")

# Re-do 'c' install, but capture stdout this time to make sure that we have a
# message indicating that 'a1' being excluded is preventing 'c' installation.
# The solvers indicate this case with different messages.
status, stdout = opkgcl.opkgcl("--force-postinstall install c --add-exclude a1")
if libsolv:
    if not re.search(r"package a1-([^ ]*) can only be installed by a direct request", stdout):
        opk.fail("[libsolv] Package 'c' install should indicate 'a1' only installable by direct request")
else:
    if not re.search(r"exclude required package a1 at users request", stdout):
        opk.fail("[internalsolv] Package 'c' install should indicate 'a1' was excluded")

opkgcl.install("b", "--add-exclude 'a*'")
if not opkgcl.is_installed("b"):
    opk.fail("Package 'b' not installed, excluded pacakge was only a recommendation.")
if opkgcl.is_installed("a1"):
    opk.fail("Package 'a1' installed despite being excluded.")
if opkgcl.is_installed("a2"):
    opk.fail("Package 'a2' installed despite being excluded.")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Make available two versions of the same package and install it. Check that the
# latest version is chosen out of the two available. Then explicitly install the
# old version and check that it replaces the newer version, resulting in a
# downgrade.
#

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a", Version="1.0")
o.add(Package="a", Version="2.0")
o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.install("a")
if not opkgcl.is_installed("a", "2.0"):
    opk.fail("Package 'a' installed but latest version does not report as installed.")
if opkgcl.is_installed("a", "1.0"):
    opk.fail("Package 'a' version '1.0' installed as well as version '2.0'.")

# Explicitly install old version
opkgcl.install("a_1.0_all.opk", "--force-downgrade")
if not opkgcl.is_installed("a", "1.0"):
    opk.fail("Package 'a' explicitly downgraded but old version does not report as installed.")
if opkgcl.is_installed("a", "2.0"):
    opk.fail("Package 'a' explicitly downgraded but latest version still reports as installed.")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Test that --download-only flag doesn't change the state
# of the system during install/remove/upgrade operations
#

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a", Version="1.0", Depends="b")
o.add(Package="b")
o.write_opk()
o.write_list()
opkgcl.update()
opkgcl.install("a")

o = opk.OpkGroup()
o.add(Package="a", Version="2.0", Depends="c")
o.add(Package="c")
o.write_opk()
o.write_list()

opkgcl.update()

status = opkgcl.install("c", "--download-only")
if opkgcl.is_installed("c"):
    opk.fail("Pacakge c was installed during a --download-only operation")

status = opkgcl.remove("a", "--download-only")
if not opkgcl.is_installed("a"):
    opk.fail("Package a was removed during a --download-only operation")

status = opkgcl.upgrade("--download-only")
if not opkgcl.is_installed("a", "1.0"):
    opk.fail("Pacakge a was upgraded during a --download-only operation")
if not opkgcl.is_installed("b"):
    opk.fail("Package b was incorrectly removed during a --download-only operation")
if opkgcl.is_installed("c"):
    opk.fail("Package c was incorrectly installed during a --download-only operation")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Install version 1.0 of a package 'x' which PROVIDES and CONFLICTS 'v', then
# try to upgrade it.

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="x", Provides="v", Conflicts="v", Version="1.0")
o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.install("x")
if not opkgcl.is_installed("x"):
    opk.fail("Package 'x' installed but reports as not installed.")

# Now try to upgrade

o.add(Package="x", Provides="v", Conflicts="v", Version="2.0")
o.write_opk()
o.write_list()

opkgcl.update()
opkgcl.upgrade()

if not opkgcl.is_installed("x", "2.0"):
    opk.fail("New version of package 'x' available during upgrade but was not installed")
if opkgcl.is_installed("x", "1.0"):
    opk.fail("Package 'a' upgraded but old version still installed.")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Reporter: Graham Gower
#
# If a tarball contains a long symlink (over 100 chars) in a longpath (over 100
# chars) then the resulting link or path can be truncated to 100 chars.
#
# This is due to a bug where if both 'L' and 'K' entries are found in the
# tarball, only the first one takes affect due to get_header_tar recursively
# calling itself. To fix this, process longname and linkname at the end of the
# function rather than the start after any subcalls have taken place.
#
#
# Status
# ======
#
# Fixed in r594.

import os
import opk, cfg, opkgcl

opk.regress_init()

long_dir = 110*"a"
long_b = 110*"b"
long_filename = long_dir + "/"+ long_b
long_filename2 = long_dir + "/" + 110*"c"

os.mkdir(long_dir)
open(long_filename, "w").close()
os.symlink(long_b, long_filename2)
a = opk.Opk(Package="a")
a.write(data_files=[long_dir, long_filename, long_filename2])
os.unlink(long_filename)
os.unlink(long_filename2)
os.rmdir(long_dir)
opkgcl.install("a_1.0_all.opk")

if not opkgcl.is_installed("a"):
    opk.fail("Package 'a' not installed.")

if not os.path.exists("{}/{}".format(cfg.offline_root, long_dir)):
    opk.fail("dir with name longer than 100 characters not created.")

if not os.path.exists("{}/{}".format(cfg.offline_root, long_filename)):
    opk.fail("file with a name longer than 100 characters, "
             "in dir with name longer than 100 characters, "
             "not created.")

if not os.path.lexists("{}/{}".format(cfg.offline_root, long_filename2)):
    opk.fail("symlink with a name longer than 100 characters, "
             "pointing at a file with a name longer than "
             "100 characters,"
             "in dir with name longer than 100 characters, "
             "not created.")

linky = os.path.realpath("{}/{}".format(cfg.offline_root, long_filename2))
linky_dst = os.path.realpath("{}/{}".format(cfg.offline_root, long_filename))
if linky != linky_dst:
    opk.fail("symlink path truncated.")

opkgcl.remove("a")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Install a package B, which recommends package A. Verify that A installs
# automatically and gets flagged as auto-installed. Make a newer version
# of A and B available. Do upgrade with --autoremove option and ensure
# that A gets uninstalled correctly instead of being upgraded.
#
# This test case matches issue 144.
#

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a", Version="1.0")
o.add(Package="b", Version="1.0", Recommends="a")
o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.install("b")
if not opkgcl.is_installed("a"):
    opk.fail("Package a should have been auto-installed")
if not opkgcl.is_autoinstalled("a"):
    opk.fail("Package a should have been marked as auto-installed")

o = opk.OpkGroup()
o.add(Package="a", Version="2.0")
o.add(Package="b", Version="2.0")
o.write_opk()
o.write_list()

opkgcl.update()
opkgcl.upgrade("b", "--autoremove")

if opkgcl.is_installed("a"):
    opk.fail("Package a should have been auto-removed")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Create 'a' which depends on 'b' later than or equal to 2.0.
# create 'b' in versions 1.0, 2.0, and 3.0.
#
# Install 'a' and check that 'b(3.0)' is installed.
#
# Try to downgrade 'b' to version 1.0 and check that it fails.
#
# Try to upgrade 'b' to version 2.0 and check that it is allowed.
#

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()

o.add(Package="a", Depends="b (>= 2.0)")
o.add(Package="b", Version="1.0")
o.add(Package="b", Version="2.0")
o.add(Package="b", Version="3.0")

o.write_opk()
o.write_list()

opkgcl.update()

# install a and check that b(3.0) is installed
opkgcl.install("a")
if not opkgcl.is_installed("a"):
    opk.fail("Package 'a' installed but reports as not installed.")
if opkgcl.is_installed("b", "1.0"):
    opk.fail("Package 'a' depends on 'b' later or equal to 2.0, but 'b(1.0)' installed.")
if opkgcl.is_installed("b", "2.0"):
    opk.fail("Package 'b(3.0)' available, but 'b(2.0)' installed.")
if not opkgcl.is_installed("b", "3.0"):
    opk.fail("Package 'a' depends on 'b' later than or equal to 2.0, but 'b' not installed.")

# try forcing a downgrade to b(1.0) which should fail
opkgcl.install("b_1.0_all.opk", "--force-downgrade")
if not opkgcl.is_installed("a"):
    opk.fail("Package 'a' was removed.")
if opkgcl.is_installed("b", "2.0"):
    opk.fail("Install called on 'b(1.0)' but 'b' downgraded to(2.0).")
if opkgcl.is_installed("b", "1.0"):
    opkg.fail("Package 'a' depends on 'b' later than or equal to 2.0, but 'b' downgraded to 1.0.")
if not opkgcl.is_installed("b", "3.0"):
    opk.fail("Package 'a' depends on 'b' later than or equal to 2.0, but 'b(3.0)' removed.")

# try forcing a downgrade to b(2.0) which should be allowed
opkgcl.install("b_2.0_all.opk","--force-downgrade")
if not opkgcl.is_installed("a"):
    opk.fail("Package 'a' was removed.")
if opkgcl.is_installed("b", "1.0"):
    opk.fail("Install called on 'b(2.0)' with --force-downgrade but 'b' downgraded to 1.0.")
if opkgcl.is_installed("b", "3.0"):
    opkg.fail("Install called on 'b(2.0)' with --force-downgrade but 'b(3.0)' still installed.")
if not opkgcl.is_installed("b", "2.0"):
    opkg.fail("Install called on 'b(2.0)' with --force-downgrade but 'b' not downgraded to 2.0.")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Reporter: scot.salmon@ni.com
#
# What steps will reproduce the problem?
# ======================================
#
# 1.- Create package 'a', which PROVIDES and CONFLICTS 'x'
# 2.- Create package 'b', which DEPENDS on 'x'
# 3.- Create package 'x'
# 4.- Install 'a'
# 5.- Install 'b'
#
# What is the expected output? What do you see instead?
# ======================================
#
# Installation of b fails (it shouldn't)
#

import os
import opk, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a", Provides="x", Conflicts="x")
o.add(Package="b", Depends="x")
o.add(Package="x")
o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.install("a")
if not opkgcl.is_installed("a"):
    opk.fail("Package 'a' installed but reports as not installed.")

opkgcl.install("b")
if not opkgcl.is_installed("b"):
        opk.fail("Package 'b' installed but reports as not installed.")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Issue 152
#
# Create a package 'a' which depends on 'p', where 'p' is provided by packages
# 'b' and 'c'. Ensure that 'a' can be installed and either 'b' or 'c' is
# installed.
#
# This ensures that opkg can select a provider when both providers are
# effectively equal. Neither is preferred, neither is held, neither matches the
# abstract package name, etc.
#
# In opkg v0.2.3 this failed: neither provider was selected and so no provider
# of 'p' was returned.
#

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a", Depends="p")
o.add(Package="b", Provides="p")
o.add(Package="c", Provides="p")
o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.install("a")
if not opkgcl.is_installed("a"):
    opk.fail("Package 'a' failed to install.")
if not (opkgcl.is_installed("b") or opkgcl.is_installed("c")):
    opk.fail("No provider of 'p' was installed.")



#! /usr/bin/env python3
#
# Create package 'a' with a nonexistent dependency.
# Install 'a' with --force-depends and check that 'a' is installed.
#
# Create package 'b' which depends on 'c'
# Install 'b' with --force-depends and check that both 'b' and 'c' are installed.
# Remove 'c' with --force-depends and check that 'c' is removed
# while 'b' remains installed
#


import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a", Depends="nonexistent")
o.add(Package="b", Depends="c")
o.add(Package="c");


o.write_opk()
o.write_list()

opkgcl.update()

# check that --force-depends works when installing a package
opkgcl.install("a","--force-depends")
if not opkgcl.is_installed("a"):
        opk.fail("Package 'a' installed with --force-depends but does not report as installed.")

# check that --force-depends does not stop deps from being installed when available
opkgcl.install("b", "--force-depends")
if not opkgcl.is_installed("b"):
    opk.fail("Package 'b' installed but does not report as installed.")
if not opkgcl.is_installed("c"):
    opk.fail("Package 'b' depends on 'c' and 'c' is available, but 'c' not installed.")

# check that --force-depends works when removing a package
opkgcl.remove("c", "--force-depends")
if opkgcl.is_installed("c"):
    opk.fail("Package 'c' removed with --force-depends but reports as still installed.")
if not opkgcl.is_installed("b"):
    opk.fail("Package 'b' not removed but reports as not installed.")



#!/usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Create package `a` which recommends `b`
# Install `a`
# Pkg 'a' and 'b' should have be installed
# No error occurs when trying to install 'b' again.

import os
import opk, cfg, opkgcl

opk.regress_init()
o = opk.OpkGroup()

o.add(Package='a', Recommends='b')
o.add(Package='b')
o.write_opk()
o.write_list()

opkgcl.update()

rc, _ = opkgcl.opkgcl('install a')

assert rc == 0, "Opkg failed when installing 'a'"

if not opkgcl.is_installed('a'):
    opk.fail("Package 'a' failed to install.")

if not opkgcl.is_installed('b'):
    opk.fail("Package 'b' (recommended by 'a') failed to install.")

rc, _ = opkgcl.opkgcl('install b')

assert rc == 0, "Opkg failed when installing 'b'"



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()

# Create a package with a test file
f = open("test_file", "w")
f.write("Test")
f.close()
pkg = opk.Opk(Package="a", Version="1.0")
pkg.write(data_files=["test_file"])
o.addOpk(pkg)
o.write_list()
os.unlink("test_file")

opkgcl.update()

# Install the package
opkgcl.install("a")
if not opkgcl.is_installed("a"):
    opk.fail("Package 'a' installed but reports as not installed.")
if not os.path.exists("%s/test_file" % cfg.offline_root):
    opk.fail("Package 'a' installed but file 'test_file' not created.")

# Search for a file in the package we added
try:
    results = opkgcl.search("*test_file*")
    if results != ["a - 1.0"]:
        opk.fail("search for file did not return 'a' as its owner")
except FileNotFoundError as f:
    opkg.fail("search for file installed by 'a' failed")
except SystemExit as s:
    # pass along any opk.fail() in try body
    raise

# Search for a file that doesn't exist
try:
    results = opkgcl.search("*does_not_exist*")
    opkg.fail("search for nonexistant file did not return fatal error code")
except FileNotFoundError as f:
    # failure is expected
    pass
except SystemExit as s:
    # pass along any opk.fail() in try body
    raise



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Reporter: alejandro.delcastillo@ni.com
#
# What steps will reproduce the problem?
# ======================================
#
# 1.- Create package 'a', which DEPENDS on 'b'
# 2.- Create package 'b', which DEPENDS on 'c'
# 3.- Create package 'c'
# 4.- Install 'a'
#
# What is the expected output? What do you see instead?
# =====================================================
#
# The order of installation should be c -> b -> a,
# instead it is b -> c -> a.
#

import os
import opk, cfg, opkgcl

opk.regress_init()
o = opk.OpkGroup()

a = opk.Opk(Package='a', Depends='b')
a.write()

# Package 'b' has a preinst script that relies
# on a file installed by package 'c'
b = opk.Opk(Package='b', Depends='c')
b.preinst = '\n'.join([
    '#!/bin/sh',
    'set e',
    'ls %s/test_file' % cfg.offline_root,
])
b.write()

f = open('test_file', 'w')
f.write('Test')
f.close()
c = opk.Opk(Package='c')
c.write(data_files=['test_file'])

o.opk_list.append(a)
o.opk_list.append(b)
o.opk_list.append(c)
o.write_list()

opkgcl.update()

os.unlink('test_file')

if opkgcl.install('a') != 0:
    opk.fail("Installation of Pacakge 'a' failed (Return value was different than 0)")
if not opkgcl.is_installed("a"):
    opk.fail("Package 'a' installed but reports as not installed.")
if not opkgcl.is_installed("b"):
    opk.fail("Package 'b' installed but reports as not installed.")
if not opkgcl.is_installed("c"):
    opk.fail("Package 'c' installed but reports as not installed.")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Issue 127 reported by muusclaus
#
# 1. call "opkg upgrade PACKAGE_NAME"
# 2. PACKAGE_NAME is a not installed package
#
# What is the expected output?
# I expected to get an error message, that the package is not installed yet.
#
# What do you see instead?
# The package will be installed like as I do an "opkg install ..."
#

import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a")
o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.upgrade("a")
if opkgcl.is_installed("a"):
    opk.fail("Package 'a' installed by upgrade.")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Create package 'a(1.0)' with short description only.
# Create package 'b(1.0)' with a complete multiline description.
# Check that info command for 'a' always includes the full description.
# Check that info command for 'b' depends on the short-description flag.
#

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a", Version="1.0", Description="Just Short")
o.add(Package="b", Version="1.0", Description="Terse\n Verbose description\n across multiple lines")
o.write_opk()
o.write_list()

opkgcl.update()

info = opkgcl.info("a")
if "Just Short" not in info:
        opk.fail("Package 'a' description should contain 'Just Short'")

info = opkgcl.info("a", flags="--short-description")
if "Just Short" not in info:
        opk.fail("Package 'a' description should contain 'Just Short'")

info = opkgcl.info("b")
if "Terse" not in info:
        opk.fail("Package 'b' description should contain 'Terse'")
if "Verbose description\n across multiple lines" not in info:
        opk.fail("Package 'b' description should contain all lines because short-description was not specified")

info = opkgcl.info("b", flags="--short-description")
if "Terse" not in info:
        opk.fail("Package 'b' description should contain 'Terse'")
if "Verbose description\n across multiple lines" in info:
        opk.fail("Package 'b' description should contain only the first line because short-description was specified")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Reporter: alexeytech@gmail.com
#
# What steps will reproduce the problem?
# ======================================
#
# 1. Create package a that have files /x1 /x2
#        install it
# 2. Create package b that have files /x2 /x3 and Replaces a, Provides a,
#    Conflicts a
#        install it
#
# What is the expected output? What do you see instead?
# =====================================================
#
# a is removed, b is installed as expected, BUT opkg files b shows only file
# /x3 and /x2 stays in filesystem untracked. I expect that package B must own
# file a/x2.
#
# Status: Accepted
#

import os
import opk, cfg, opkgcl

opk.regress_init()

x1 = "x1"
x2 = "x2"
x3 = "x3"

open(x1, "w").close()
open(x2, "w").close()
a = opk.Opk(Package="a")
a.write(data_files=[x1, x2])
os.unlink(x1)
os.unlink(x2)
opkgcl.install("a_1.0_all.opk")

if not opkgcl.is_installed("a"):
    opk.fail("Package 'a' not installed.")

open(x2, "w").close()
open(x3, "w").close()
a = opk.Opk(Package="b", Replaces="a", Provides="a", Conflicts="a")
a.write(data_files=[x2, x3])
os.unlink(x2)
os.unlink(x3)
opkgcl.install("b_1.0_all.opk")

if not opkgcl.is_installed("b"):
        opk.fail("Package 'b' not installed.")
if opkgcl.is_installed("a"):
        opk.fail("Package 'a' was not replaced.")

x2fullpath = "{}/x2".format(cfg.offline_root)
if not x2fullpath in opkgcl.files("b"):
        opk.fail("Package 'b' does not own file 'x2'.")

opkgcl.remove("b")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Create essential packages 'a(1.0)' and 'a(2.0)'.
# Install 'a(1.0)'.
# Remove 'a' and check that 'a' is not removed because it is essential
# Upgrade 'a' to version (2.0) and check that the upgrade works
# Remove 'a' and check that 'a' is not removed because it is essential
# Remove 'a' with --force-removal-of-essential-packages and check that it is removed
#

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a", Essential="yes")
o.add(Package="a", Version = "2.0", Essential="yes")

o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.install("a_1.0_all.opk")
if not opkgcl.is_installed("a","1.0"):
        opk.fail("Package 'a' installed but does not report as installed.")

opkgcl.remove("a")
if not opkgcl.is_installed("a"):
        opk.fail("Package 'a 1.0' is marked essential but was removed.")

# Upgrading a should be allowed
opkgcl.install("a_2.0_all.opk")
if not opkgcl.is_installed("a","2.0"):
        opk.fail("Package 'a 2.0' available but was not upgraded.")
if opkgcl.is_installed("a","1.0"):
        opk.fail("Package 'a' upgraded but old version not removed.")

opkgcl.remove("a")
if not opkgcl.is_installed("a"):
        opk.fail("Package 'a 2.0' is marked essential but was removed.")

opkgcl.remove("a", "--force-removal-of-essential-packages")
if opkgcl.is_installed("a"):
        opk.fail("Package 'a' was removed with"
                 " --force-removal-of-essential-packages but reports as installed")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Reported: alejandro.delcastillo@ni.com
#
# What steps will reproduce the problem?
########################################
#   1. Create package 'a', which PROVIDES and CONFLICTS 'b'
#   2. Create package 'b'.
#   3. Install 'b'
#   4. Install 'a'
#
# What is the expected output? What do you see instead?
########################################
#
#   Installation of 'a' should fail, since 'a' conflicts with 'b'.
#

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a", Provides="b", Conflicts="b")
o.add(Package="b")
o.write_opk()
o.write_list()

opkgcl.update()
opkgcl.install("b")

if not opkgcl.is_installed("b"):
    opk.fail("Package 'b' failed to install.")

opkgcl.install("a")
if opkgcl.is_installed("a"):
    opk.fail("Package 'a' was incorrectly installed, since it conflicts with b.")



#! /usr/bin/env python3
#
# Create package 'a(1.0)' with 2 other fields.
# Check that info command reports all fields without a filter.
# Check that info command reports specified fields only with a filter.
#

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a", Version="1.0", Description="d", Section="s")
o.write_opk()
o.write_list()

opkgcl.update()

info = opkgcl.info("a")
if "Package" not in info:
        opk.fail(info)
if "Version" not in info:
        opk.fail("Package 'a' information should contain 'Version' field")
if "Description" not in info:
        opk.fail("Package 'a' information should contain 'Description' field")
if "Section" not in info:
        opk.fail("Package 'a' information should contain 'Section' field")
if "Essential" in info:
        opk.fail("Package 'a' information should not contain 'Essential' field because it is not in the package")

info = opkgcl.info("a", "Version,Description,SomethingElse")
if "Package" not in info:
        opk.fail("Package 'a' information should contain 'Package' field even if it's not in the filter")
if "Version" not in info:
        opk.fail("Package 'a' information should contain 'Version' field because it is in the filter")
if "Description" not in info:
        opk.fail("Package 'a' information should contain 'Description' field because it is in the filter")
if "Section" in info:
        opk.fail("Package 'a' information should not contain 'Section' field because it is not in the filter")

info = opkgcl.info("a", "SomethingElse")
if "Package" not in info:
        opk.fail("Package 'a' information should contain 'Package' field even if it's not in the filter")
if "Version" in info:
        opk.fail("Package 'a' information should not contain 'Version' field because it is not in the filter")
if "Description" in info:
        opk.fail("Package 'a' information should not contain 'Description' field because it is not in the filter")
if "Section" in info:
        opk.fail("Package 'a' information should not contain 'Section' field because it is not in the filter")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Reported by ccospel, Jan 27, 2012
#
# What steps will reproduce the problem?
#   1. Create two packages that provide the same virtual package.
#      - Package A should provide X
#      - Package B should provide X, replace X, conflict X
#   2. Install package A.  Then install package B.
#   3. Package B will be installed successfully, but package A will not have
#      first been removed. If doing an "opkg list", both packages appear and
#      package A files still exist in the file system.
#
# What is the expected output? What do you see instead?
#   I expect package A to first be removed, and then for package B to be
#   installed.  Instead, it remains and its files are overwritten by package
#   B's.

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a", Provides="x")
o.add(Package="b", Provides="x", Replaces="x", Conflicts="x")
o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.install("a")
if not opkgcl.is_installed("a"):
    opk.fail("Package 'a' failed to install.")

opkgcl.install("b")
if not opkgcl.is_installed("b"):
    opk.fail("Package 'b' failed to install.")
if opkgcl.is_installed("a"):
    opk.fail("Package 'a' remains installed despite conflicts.")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Reporter: Graham Gower
#
# Denys Dmytriyenko has experienced problems using BAD_RECOMMENDATAIONS with OE.
# See post here:
# http://lists.linuxtogo.org/pipermail/openembedded-devel/2010-February/017648.html
#
#
# Status
# ======
#
# Graham Gower:
# > Should be fixed with r553.

import os
import opk, cfg, opkgcl
vardir=os.environ['VARDIR']

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a", Version="1.0", Recommends="b")
o.add(Package="b", Version="2.0")
o.write_opk()
o.write_list()

# prime the status file so 'b' is not installed as a recommendation
os.makedirs(("{}"+vardir+"/lib/opkg").format(cfg.offline_root))
status_filename = ("{}"+vardir+"/lib/opkg/status").format(cfg.offline_root)
f = open(status_filename, "w")
f.write("Package: b\n")
f.write("Version: 1.0\n")
f.write("Architecture: all\n")
f.write("Status: deinstall hold not-installed\n")
f.close()

opkgcl.update()

opkgcl.install("a")
if opkgcl.is_installed("b"):
    opk.fail("Package 'b' installed despite deinstall/hold status.")

opkgcl.remove("a")
opkgcl.install("a")
if opkgcl.is_installed("b"):
    opk.fail("Package 'b' installed - deinstall/hold status not retained.")

opkgcl.remove("a")
open(status_filename, "w").close()



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Create 'a' which depends on 'b' later than 2.0.
# create 'b' in versions 1.0, 2.0, and 3.0.
#
# Install 'a' and check that 'b(3.0)' is installed.
#
# Try to downgrade 'b' to version 2.0 and check that it fails
#

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()

o.add(Package="a", Depends="b (>> 2.0)")
o.add(Package="b", Version="1.0")
o.add(Package="b", Version="2.0")
o.add(Package="b", Version="3.0")

o.write_opk()
o.write_list()

opkgcl.update()

# install a and check that b(3.0) is installed
opkgcl.install("a")
if not opkgcl.is_installed("a"):
    opk.fail("Package 'a' installed but reports as not installed.")
if opkgcl.is_installed("b", "1.0"):
    opk.fail("Package 'a' depends on 'b' later than 2.0, but 'b(1.0)' installed.")
if opkgcl.is_installed("b", "2.0"):
    opk.fail("Package 'a' depends on 'b' later than 2.0, but 'b(2.0)' installed.")
if not opkgcl.is_installed("b", "3.0"):
    opk.fail("Package 'a' depends on 'b' later than 2.0, but 'b(3.0)' not installed.")

# try downgrading to b(2.0) which should fail
opkgcl.install("b_2.0_all.opk","--force-downgrade")
if not opkgcl.is_installed("a"):
    opk.fail("Package 'a' was removed.")
if opkgcl.is_installed("b", "1.0"):
    opk.fail("Install called on 'b(2.0)', but 'b(1.0)' installed.")
if opkgcl.is_installed("b", "2.0"):
    opk.fail("Package 'a' depends on 'b' later than 2.0, but 'b' downgraded to 2.0.")
if not opkgcl.is_installed("b", "3.0"):
    opk.fail("Package 'a' depends on 'b' later than 2.0, but 'b(3.0)' removed.")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Reporter: alejandro.delcastillo@ni.com
#
# What steps will reproduce the problem?
# ======================================
#
# 1.- Create package a, which depends on b and c
# 2.- Create package b
# 3.- Create package c, which depends on b
# 4.- Install a
# 5.- Remove a with --autoremove
#
# What is the expected output? What do you see instead?
# =====================================================
#
# All packages should be removed. Instead, b is left behind.
#

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a", Depends="b, c")
o.add(Package="b")
o.add(Package="c", Depends="b")

o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.install("a")
if not opkgcl.is_installed("a"):
        opk.fail("Package 'a' installed but does not report as installed.")
if not opkgcl.is_installed("b"):
        opk.fail("Package 'b' should be installed as a dependency of 'a' but does not report as installed.")
if not opkgcl.is_installed("c"):
        opk.fail("Package 'c' should be installed as a dependency of 'a' but does not report as installed.")

# Check the packages are marked correctly
if opkgcl.is_autoinstalled("a"):
    opk.fail("Package 'a' explicitly installed by user but reports as auto installed.")
if not opkgcl.is_autoinstalled("b"):
    opk.fail("Package 'b' installed as a dependency but does not report as auto installed.")
if not opkgcl.is_autoinstalled("c"):
    opk.fail("Package 'c' installed as a dependency but does not report as auto installed.")

# Check that autoinstalled packages are removed properly
opkgcl.remove("a","--autoremove")
if opkgcl.is_installed("a"):
        opk.fail("Package 'a' removed but reports as installed.")
if opkgcl.is_installed("b"):
        opk.xfail("[internalsolv] Package 'b' not removed from --autoremove.")
if opkgcl.is_installed("c"):
        opk.fail("Package 'c' not removed from --autoremove.")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Reporter: Eric Yu
#
# What steps will reproduce the problem?
# ======================================
#
# 1. Create packages 'x', 'a', and 'b' such that 'a' recommends 'b'
#    and 'b' conflicts 'x'.
# 2. Install 'x'.
# 3. Try to install 'a'.
#
# What is the expected output? What do you see instead?
# =====================================================
#
# 'b' should remain uninstalled, but 'a' should be installed because 'b' is not
# an absolute dependency of 'a'.
#
# Instead nothing is  installed.
#
# Status
# ======
# Open
#

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()

o.add(Package="x")
o.add(Package="a", Recommends="b")
o.add(Package="b", Conflicts="x")

o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.install("x");
if not opkgcl.is_installed("x"):
    opk.fail("Package 'x' installed but reports as not installed")

opkgcl.install("a");
if not opkgcl.is_installed("x"):
    opk.fail("Package 'x' removed when 'a' installed, but should not be removed.")
if opkgcl.is_installed("b"):
    opk.fail("Package 'b' installed but conflicts with x.")
if not opkgcl.is_installed("a"):
    opk.xfail("[internalsolv] Package 'a' not installed even though 'b' is not an absolute dependency of 'a'.")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Test opkg when specifying the version to install via:
#       opkg install <pkg_name>=<pkg_version>
#
# 1.- Create packages 'a(1.0)', 'a(2.0)' and 'a(3.0)'
# 2.- Install 'a=1.0'
# 3.- Upgrade
# 4.- Install 'a=2.0', with --force-downgrade
# 5.- Install 'a=2.0', with --force-reinstall
#

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a", Version ="1.0")
o.add(Package="a", Version ="2.0")
o.add(Package="a", Version ="3.0")
o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.install("a=1.0")
if not opkgcl.is_installed("a", "1.0"):
    opk.fail("Package 'a', Version '1.0', failed to install.")

opkgcl.upgrade("a")
if not opkgcl.is_installed("a", "3.0"):
    opk.fail("Package 'a' was not upgraded to Version '3.0'.")

opkgcl.install("a=2.0", "--force-downgrade")
if not opkgcl.is_installed("a", "2.0"):
    opk.fail("Package 'a' was not downgraded to Version '2.0'.")

opkgcl.install("a=2.0", "--force-reinstall")
if not opkgcl.is_installed("a", "2.0"):
    opk.fail("Package 'a', Version '2.0 was not re-installed.")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Reporter: pixdamix@gmail.com
#
# What steps will reproduce the problem?
# ======================================
#
# 1. Install a package A which depends on B
# 2. Update the package A which now depends on C, but C conflicts with B
# 3. Upgrade
#
# What is the expected output? What do you see instead?
# =====================================================
#
# Expected:
#
# root@terminal:# opkg upgrade -force-defaults -autoremove
# Upgrading A on root from 1.0-r0.1 to 2.0-r0.1...
# Downloading ...
# Installing C (1.0-r0) to root...
# The following packages conflict with C: B
# B is now orphaned, uninstalling...
#
# Actual:
#
# root@terminal:# opkg upgrade -force-defaults -autoremove
# Upgrading A on root from 1.0-r0.1 to 2.0-r0.1...
# Downloading ...
# Installing C (1.0-r0) to root...
# Collected errors: * ERROR: The following packages conflict with C: *
#
# Status: Accepted
#

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a", Version="1.0", Depends="b")
o.add(Package="b")
o.write_opk()
o.write_list()
opkgcl.update()
opkgcl.install("a")

o = opk.OpkGroup()
o.add(Package="a", Version="2.0", Depends="c")
o.add(Package="c", Conflicts="b")
o.write_opk()
o.write_list()

opkgcl.update()
status = opkgcl.upgrade("a", "--autoremove")

if not opkgcl.is_installed("a", "2.0"):
    opk.xfail("[internalsolv] New version of package 'a' available during upgrade but was not installed")
if status != 0:
    opk.fail("Return value was different than 0")

opkgcl.remove("a")
opkgcl.remove("b")
opkgcl.remove("c")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Mark an installed package a-1.0 as on hold, make available a-2.0 depended on
# by z. Check that z refuses to install due to a-1.0 being held.

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="x", Version="1.0")
o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.install("x")
if not opkgcl.is_installed("x"):
    opk.fail("Package 'x' installed but reports as not installed.")

opkgcl.flag_hold("x")

# Make new versions of 'x' and 'z' available
o.add(Package="x", Version="2.0")
o.add(Package="z", Depends="x>=2.0")
o.write_opk()
o.write_list()

opkgcl.update()
opkgcl.install("z")

if opkgcl.is_installed("z"):
    opk.fail("Package 'z' installed despite dependency on 'x>=2.0' with 'x' version 1.0 on hold.")
if opkgcl.is_installed("x", "2.0"):
    opk.fail("Old version of package 'x' flagged as on hold but was upgraded.")
if not opkgcl.is_installed("x", "1.0"):
    opk.fail("Package 'x' not upgraded but old version was removed.")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Reporter: jatinvatsalya@gmail.com
#
# What steps will reproduce the problem?
########################################
#   1. Create package 'a', version 2.0.
#   2. Create package 'b', which DEPENDS on 'a', version 1.0
#   3. Install 'a' and 'b' together (b won't install)
#   4. Remove 'a'
#   4. Install 'a'
#
# What is the expected output? What do you see instead?
########################################
#   The first install operation should install 'a', but fail on 'b'. The
#   second install operation should succeed, but instead it fails.

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a", Version="2.0")
o.add(Package="b", Depends="a (=1.0)")
o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.install("a b")
# Using the internal solver, 'a' does get installed, but using libsolv, it doesn't.
# Skipping check for 'a' status so the test will work with both solvers.
# Regarding 'b', independently of the backend, it shouldn't be installed
if opkgcl.is_installed("b"):
        opk.fail("Package 'b' installed despite missing dependency on 'a' (=1.0).")

opkgcl.remove("a")
if opkgcl.is_installed("a"):
        opk.fail("Package 'a' removed but still reports as being installed")

opkgcl.install("a")
if not opkgcl.is_installed("a"):
    opk.fail("Package 'a' failed to install.")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Install a package 'x' which PROVIDES 'v' and ensure that we can install
# another package 'y' which also PROVIDES 'v'.

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="x", Provides="v")
o.add(Package="y", Provides="v")
o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.install("x")
if not opkgcl.is_installed("x"):
    opk.fail("Package 'x' installed but reports as not installed.")

# Now try to install "y", which should succeed

opkgcl.install("y")
if not opkgcl.is_installed("y"):
    opk.fail("Package 'y' not installed despite lack of conflicts.")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Reporter: pixdamix
#
# What steps will reproduce the problem?
# ======================================
#
# 1. Extract two package package-a, package-b (wich depends on package-a)
# 2. Under some condition when package are flagged as extracted
#    opkg configure will configure package-b before package-a
#
#
# What is the expected output? What do you see instead?
# =====================================================
#
# opkg should configure package-a before package-b
#
#
# Status
# ======
#
# pixdamix:
# > Resolved in r521

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a", Depends="b")
o.add(Package="b")
o.write_opk()
o.write_list()

opkgcl.update()

(status, output) = opkgcl.opkgcl("install --force-postinstall a")
ln_a = output.find("Configuring a")
ln_b = output.find("Configuring b")

if ln_a == -1:
    opk.fail("Didn't see package 'a' get configured.")

if ln_b == -1:
    opk.fail("Didn't see package 'b' get configured.")

if ln_a < ln_b:
    opk.fail("Packages 'a' and 'b' configured in wrong order.")

opkgcl.remove("a")
opkgcl.remove("b")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Reporter: alejandro.delcastillo@ni.com
#
# Test the use of globs during install and
# remove operations
#

import os
import opk, cfg, opkgcl
import re

opk.regress_init()

o = opk.OpkGroup()
o.add(Package='a-b', Version='1.0')
o.add(Package='a-c', Version='1.0')
o.add(Package='b-a', Version='1.0')
o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.install("'a*'")
if not opkgcl.is_installed('a-b'):
    opk.xfail("[internalsolv] Package 'a-b' not installed but reports as being installed.")
if not opkgcl.is_installed('a-c'):
    opk.fail("Package 'a-c' not installed but reports as being installed.")
if opkgcl.is_installed('b-a'):
    opk.fail("Package 'b-a' was incorrectly installed")

opkgcl.remove("'a*'")
if opkgcl.is_installed('a-b'):
    opk.fail("Package 'a-b' failed to remove")
if opkgcl.is_installed('a-c'):
    opk.fail("Package 'a-c' failed to remove")

opkgcl.install("'?-b'")
if not opkgcl.is_installed('a-b'):
    opk.fail("Package 'a-b' not installed but reports as being installed.")
if opkgcl.is_installed('a-c'):
    opk.fail("Package 'a-c' was incorrectly installed")
if opkgcl.is_installed('b-a'):
    opk.fail("Package 'b-a' was incorrectly installed")

opkgcl.remove("'?-b'")
if opkgcl.is_installed('a-b'):
    opk.fail("Package 'a-b' failed to remove")

opkgcl.install("a-[bc]")
if not opkgcl.is_installed('a-b'):
    opk.fail("Package 'a-b' not installed but reports as being installed.")
if not opkgcl.is_installed('a-c'):
    opk.fail("Package 'a-c' not installed but reports as being installed.")
if opkgcl.is_installed('b-a'):
    opk.fail("Package 'b-a' was incorrectly installed")

o.add(Package='a-b', Version='2.0')
o.add(Package='a-c', Version='2.0')
o.add(Package='b-a', Version='2.0')
o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.upgrade("'a*'")
if not opkgcl.is_installed('a-b', '2.0'):
    opk.fail("New version of package 'a-b' available during upgrade but was not installed.")
if not opkgcl.is_installed('a-c', '2.0'):
    opk.fail("New version of package 'a-c' available during upgrade but was not installed.")
if opkgcl.is_installed('b-a'):
    opk.fail("Package 'b-a' was incorrectly installed")

opkgcl.remove("'a*'")
opkgcl.install('b-a=1.0')

opkgcl.upgrade("'?-a'")
if not opkgcl.is_installed('b-a', '2.0'):
    opk.fail("New version of package 'b-a' available during upgrade but was not installed.")
if opkgcl.is_installed('a-b'):
    opk.fail("Package 'a-b' was incorrectly installed")
if opkgcl.is_installed('a-c'):
    opk.fail("Package 'a-c' was incorrectly installed")

opkgcl.remove("'*'")
opkgcl.install('a-b=1.0')
opkgcl.install('a-c=1.0')

opkgcl.upgrade("a-[bc]")
if not opkgcl.is_installed('a-b', '2.0'):
    opk.fail("New version of package 'a-b' available during upgrade but was not installed..")
if not opkgcl.is_installed('a-c', '2.0'):
    opk.fail("New version of package 'a-c' available during upgrade but was not installed..")
if opkgcl.is_installed('b-a'):
    opk.fail("Package 'b-a' was incorrectly installed")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Within the function `pkg_hash_fetch_best_installation_candidate`, we wish to
# test the condition where `(held_pkg && good_pkg_by_name) == true`. So install
# a 1.0, mark it as on hold and explicitly install 'a_2.0_all.opk'. This
# installation should be blocked.
#

import os
import opk, cfg, opkgcl

opk.regress_init()

a_10 = opk.Opk(Package="a", Version="1.0")
a_10.write()
a_20 = opk.Opk(Package="a", Version="2.0")
a_20.write()

opkgcl.install("a_1.0_all.opk")
if not opkgcl.is_installed("a"):
    opk.fail("Package 'a' installed but reports as not installed.")

opkgcl.flag_hold("a")

# Try to explicitly install a version 2.0
opkgcl.install("a_2.0_all.opk")

if opkgcl.is_installed("a", "2.0"):
    opk.fail("Old version of package 'a' flagged as on hold but was upgraded.")
if not opkgcl.is_installed("a", "1.0"):
    opk.fail("Package 'a' not upgraded but old version was removed.")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Reporter: Graham Gower
#
# What steps will reproduce the problem?
# ======================================
#
# 1. Create package a version 1, containing file /foo.
# 2. Create package a version 2, without file /foo, depending on package b.
# 3. Create package b version 1, containing file /foo which Conflicts  a (<< 2.0)
# 4. Create package repository containing a_2 and b_1.
# 5. Install a_1 manually.
# 6. opkg update; opkg upgrade
#
#
# What is the expected output? What do you see instead?
# =====================================================
#
# File /foo is expected to exist and be provided by package b_1. This is not the
# case, file /foo has disappeared.
#
# opkg_remove.c: remove_data_files_and_list() should double check that the file
# belongs to the package before removing it.
#
#
# Status
# ======
#
# Fixed by r531-r534,r536

import os
import opk, cfg, opkgcl

opk.regress_init()

open("foo", "w").close()
a1 = opk.Opk(Package="a", Version="1.0")
a1.write(data_files=["foo"])

opkgcl.install("a_1.0_all.opk")

o = opk.OpkGroup()
a2 = opk.Opk(Package="a", Version="2.0", Depends="b")
a2.write()
b1 = opk.Opk(Package="b", Version="1.0", Conflicts="a (<< 2.0)", Replaces="a (<< 2.0)")
b1.write(data_files=["foo"])
o.opk_list.append(a2)
o.opk_list.append(b1)
o.write_list()

os.unlink("foo")

opkgcl.update()
status = opkgcl.upgrade("a")

if status != 0:
        opk.fail("Upgrade operation failed (Return value was different than 0)")

if not opkgcl.is_installed("a", "2.0"):
    opk.fail("Package 'a_2.0' not installed.")

foo_fullpath = "{}/foo".format(cfg.offline_root)

if not os.path.exists(foo_fullpath):
    opk.fail("File 'foo' incorrectly orphaned.")

if not foo_fullpath in opkgcl.files("b"):
    opk.fail("Package 'b' does not own file 'foo'.")

opkgcl.remove("a")
opkgcl.remove("b")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Reported by boris.bischoff, Aug 17, 2012
#
# What steps will reproduce the problem?
#   1. Install a package with "opkg install" that contains symlinks to
#      directories which will be created.
#   2. Remove the very same package with "opkg remove".
#
# What is the expected output? What do you see instead?
#   I expect all files/directories to be deleted, even the symlinks.
#   Instead, all files/directories are deleted except for the symlinks.
#

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()

os.mkdir("test_dir")
os.symlink("test_dir", "test_link")
pkg = opk.Opk(Package="a", Version="1.0")
pkg.write(data_files=["test_link", "test_dir"])
o.addOpk(pkg)
o.write_list()
os.unlink("test_link")
os.rmdir("test_dir")

opkgcl.update()

opkgcl.install("a")
if not opkgcl.is_installed("a"):
    opk.fail("Package 'a' failed to install.")
if not os.path.exists("%s/test_dir" % cfg.offline_root):
    opk.fail("Package 'a' installed but 'test_dir' not created")
if not os.path.lexists("%s/test_link" % cfg.offline_root):
    opk.fail("Package 'a' installed but 'test_link' not created")

opkgcl.remove("a")
if opkgcl.is_installed("a"):
    opk.fail("Package 'a' failed to remove.")
if os.path.exists("%s/test_dir" % cfg.offline_root):
    opk.fail("Package 'a' removed but 'test_dir' not deleted")
if os.path.lexists("%s/test_link" % cfg.offline_root):
    opk.fail("Package 'a' removed but 'test_link' not deleted")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only

import os
import opk, cfg, opkgcl

opk.regress_init()

open("asdf", "w").close()
a = opk.Opk(Package="a", Version="1.0", Architecture="all")
a.write(data_files=["asdf"])
b = opk.Opk(Package="b", Version="1.0", Architecture="all")
b.write(data_files=["asdf"])
os.unlink("asdf")
opkgcl.install("a_1.0_all.opk")

if not opkgcl.is_installed("a"):
    opk.fail("Package 'a' not installed.")

if not os.path.exists("{}/asdf".format(cfg.offline_root)):
    opk.fail("asdf not created.")

opkgcl.install("b_1.0_all.opk", "--force-overwrite")

if "{}/asdf".format(cfg.offline_root) not in opkgcl.files("b"):
    opk.fail("asdf not claimed by ``b''.")

if "{}/asdf".format(cfg.offline_root) in opkgcl.files("a"):
    opk.fail("asdf is still claimed by ``a''.")

opkgcl.remove("b")
opkgcl.remove("a")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Reporter: Graham Gower
#
# Opkg should try subsequent satisfying dependencies in the event of failing to
# install the first one (e.g. due to missing dependency or conflict).
#
#
# Status
# ======
#
# Open

import opk, cfg, opkgcl

def cleanup():
    opkgcl.remove("a")
    opkgcl.remove("b")
    opkgcl.remove('c')

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a", Provides="v")
o.add(Package="b", Provides="v", Depends="b_nonexistant")
o.add(Package="c", Depends="v")

o.write_opk()
o.write_list()

opkgcl.update()

# install ``c'' from repository
opkgcl.install("c")
if not opkgcl.is_installed("c"):
    cleanup()
    opk.xfail("[internalsolv] package ``c'' not installed.")

cleanup()



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Install a package containing a file and check that the file is created. Remove
# the package and check that the file is removed.
#

import os
import opk, cfg, opkgcl

opk.regress_init()

# Create data file
f = open("test_file", "w")
f.write("Test")
f.close()

# Create package including test file
o = opk.OpkGroup()
pkg = opk.Opk(Package="a")
pkg.write(data_files=["test_file"])
o.addOpk(pkg)

o.write_list()

opkgcl.update()

# Delete data file used to create opk as it will be re-created when the package
# is installed
os.unlink("test_file")

# Test...
opkgcl.install("a")
if not opkgcl.is_installed("a"):
    opk.fail("Package 'a' installed but reports as not installed.")
if not os.path.exists("%s/test_file" % cfg.offline_root):
    opk.fail("Package 'a' installed but file 'test_file' not created.")

opkgcl.remove("a")
if opkgcl.is_installed("a"):
    opk.fail("Package 'a' removed but still reports as being installed.")
if os.path.exists("%s/test_file" % cfg.offline_root):
    opk.fail("Package 'a' removed but file 'test_file' still present.")



#!/usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Reporter: Paul Spangler <paul.spangler@ni.com>
#
# What steps will reproduce the problem?
########################################
#
# 1. Create a sub-directory 'ipks' beneath the root of the feed.
# 2. Create package 'a' in the 'ipks/' directory.
# 3. Create a Package file at the feed root with an entry for package 'a'.
# 4. Set the 'Filename' attribute for 'a' to its relative path under the
#    subdirectory, like: './ipks/a_1.0.ipk'
# 5. From a CWD outside of the feed root, `opkg download a`
#
# What is the expected output? What do you see instead?
########################################
# Expected:
# Package 'a' is downloaded to the current working directory, with a relative
# path like './a_1.0.ipk'.
#
# Regressive Behavior:
# Package 'a' is downloaded to the full, relative path asserted by 'Filename'.
# opkg throws an error if the directory tree does not exist, relative to the
# current working directory.

import os
import opk, cfg, opkgcl
from shutil import rmtree
from pathlib import Path

opk.regress_init()

o = opk.OpkGroup()
o.add(Package='a', subdirectory='./ipks/')
o.write_opk()
o.write_list()
opkgcl.update()

workdir = Path(cfg.opkdir, 'work')
# refresh the work directory, clearing old downloads
try:
    rmtree(workdir)
except FileNotFoundError: pass
os.makedirs(workdir, exist_ok=False)
os.chdir(workdir)

# check that the relative-pathed package can be installed
opkgcl.install('a')
if not opkgcl.is_installed('a'):
    opk.fail("Relatively-pathed package 'a' reports as not installed.")
opkgcl.remove('a')

# check that a download operation deposits the file in work/, rather than a
# subdirectory.
status = opkgcl.download('a')
if status > 0:
    opk.fail('opkg download operation returned status code %d' % status)

path_actual = os.path.join(workdir, 'a_1.0_all.opk')
try:
    os.stat(path_actual)
except FileNotFoundError:
    opk.fail("Package 'a' downloaded to an incorrect path.")


#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Create packages 'a', 'b' which recommends 'a' and 'c' which depends on 'a'.
# Install 'b' with '--no-install-recommends' and ensure 'a' is not installed.
# Install 'c' with '--no-install-recommends' and ensure 'a' is installed.
#

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a")
o.add(Package="b", Recommends="a")
o.add(Package="c", Depends="a")
o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.install("b", "--no-install-recommends")
if opkgcl.is_installed("a"):
    opk.fail("Recommended package 'a' installed despite '--no-install-recommends'.")

opkgcl.install("c", "--no-install-recommends")
if not opkgcl.is_installed("a"):
    opk.fail("Required package 'a' not installed with '--no-install-recommends'.")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Install a package and check that it is listed as installed. Then remove the
# package and check that it is no longer listed as installed.
#

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a")
o.write_opk()
o.write_list()

opkgcl.update()

if opkgcl.is_installed("a"):
    opk.fail("Package 'a' not installed but reports as being installed.")

opkgcl.install("a")
if not opkgcl.is_installed("a"):
    opk.fail("Package 'a' installed but reports as not installed.")

opkgcl.remove("a")
if opkgcl.is_installed("a"):
    opk.fail("Package 'a' removed but still reports as being installed.")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# 1.- Create package 'a', 'b' and 'c', which install files to
#     a shared new directory.
# 2.- Install package 'b', followed by packages 'a' and 'c'.
# 3.- Remove package 'c', followed by packages 'b' and 'a'.
#
# The shared new directory should be deleted only when the last package
# 'a' is uninstalled.
#
# 4. Install and remove package 'c' again
#
# The new directory should be deleted correctly after 'c' is uninstalled.
#


import os
import opk, cfg, opkgcl

opk.regress_init()
o = opk.OpkGroup()

dir = "dir10461"

os.makedirs(dir)
a = opk.Opk(Package="a")
a.write(data_files=[dir])
o.addOpk(a)

b = opk.Opk(Package="b")
b.write(data_files=[dir])
o.addOpk(b)

c = opk.Opk(Package="c")
c.write(data_files=[dir])
o.addOpk(c)

os.rmdir(dir)

o.write_list()
opkgcl.update()

opkgcl.install("b")
if not opkgcl.is_installed("b"):
     opk.fail("Package 'b' installed but reports as not installed.")

opkgcl.install("a")
if not opkgcl.is_installed("a"):
     opk.fail("Package 'a' installed but reports as not installed.")

opkgcl.install("c")
if not opkgcl.is_installed("c"):
     opk.fail("Package 'c' installed but reports as not installed.")

opkgcl.remove("c")
if not os.path.exists("{}/{}".format(cfg.offline_root, dir)):
    opk.fail("Dir '{}' incorrectly deleted.".format(dir))

opkgcl.remove("b")
if not os.path.exists("{}/{}".format(cfg.offline_root, dir)):
    opk.fail("Dir '{}' incorrectly deleted.".format(dir))

opkgcl.remove("a")
if os.path.exists("{}/{}".format(cfg.offline_root, dir)):
    opk.fail("Dir '{}' incorrectly orphaned.".format(dir))


opkgcl.install("c")
if not os.path.exists("{}/{}".format(cfg.offline_root, dir)):
    opk.fail("Dir '{}' not created by package 'c'.".format(dir))

opkgcl.remove("c")
if os.path.exists("{}/{}".format(cfg.offline_root, dir)):
    opk.fail("Dir '{}' incorrectly orphaned.".format(dir))


#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Install package 'a' with a Depends on 'b'. Make available a new version of
# 'a' that Conflicts with 'b'. Dist-upgrade and make sure 'a' is upgraded to
# the new version, while 'b' is removed.
#

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()

o.add(Package='a', Version="1.0", Depends='b')
o.add(Package='b')
o.write_opk()
o.write_list()

opkgcl.update()
opkgcl.install('a')

if not opkgcl.is_installed('a'):
    opk.fail("Package 'a' installed but reports as not installed.")
if not opkgcl.is_installed('b'):
    opk.fail("Package 'b' installed but reports as not installed.")

o.add(Package="a", Version="2.0", Conflicts='b')
o.write_opk()
o.write_list()

opkgcl.update()
opkgcl.distupgrade()

if not opkgcl.is_installed('a', '2.0'):
    opk.xfail("[internalsolv] New version of package 'a' available during dist-upgrade but was not installed")

if opkgcl.is_installed('b'):
    opk.fail("Package 'b' was not removed.")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Reporter: Gabe Jones
#
# What steps will reproduce the problem?
# ======================================
#
# 1.- Create package 'a' and 'b', which install files to
#     a shared new directory.
# 2.- Install package 'b', followed by package 'a'.
# 3.- Remove package 'b', followed by package 'a'.
#
# What is the expected output? What do you see instead?
# ======================================
#
# The shared new directory is deleted when the package 'a'
# is uninstalled.
# The shared directory is orphaned and not deleted after
# both packages are uninstalled.

import os
import opk, cfg, opkgcl

opk.regress_init()
o = opk.OpkGroup()

dir = "dir10461"
filename1 = os.path.join(dir, "myfile1")
filename2 = os.path.join(dir, "myfile2")

os.makedirs(dir)
open(filename1, "w").close()
a = opk.Opk(Package="a")
a.write(data_files=[dir])
o.addOpk(a)

os.unlink(filename1)

open(filename2, "w").close()
b = opk.Opk(Package="b")
b.write(data_files=[dir])
o.addOpk(b)

os.unlink(filename2)
os.rmdir(dir)

o.write_list()
opkgcl.update()

opkgcl.install("b")
if not opkgcl.is_installed("b"):
     opk.fail("Package 'b' installed but reports as not installed.")

opkgcl.install("a")
if not opkgcl.is_installed("a"):
     opk.fail("Package 'a' installed but reports as not installed.")

opkgcl.remove("b")
opkgcl.remove("a")


if os.path.exists("{}/{}".format(cfg.offline_root, dir)):
    opk.fail("Dir '{}' incorrectly orphaned.".format(dir))



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Reporter: pixdamix
#
# What steps will reproduce the problem?
# ======================================
#
# 1. Given three packages A, B, C
#    A depends on B
#    A failed postinst and is in unpacked state
#    C depends on B
#
# 2. Upgrade to a new version of C which do not depends on B anymore, and use
#    --autoremove
#
#
# What is the expected output? What do you see instead?
# =====================================================
#
# B should not be removed, bot opkg uninstall it.
#
#
# Status
# ======
#
# Fixed in r625.

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a", Version="1.0", Depends="b")
o.add(Package="b", Version="1.0")
o.add(Package="c", Version="1.0", Depends="b")
o.write_opk()
o.write_list()

opkgcl.update()
opkgcl.install("a")
opkgcl.install("c")

opkgcl.flag_unpacked("a")

o = opk.OpkGroup()
o.add(Package="a", Version="1.0", Depends="b")
o.add(Package="b", Version="1.0")
o.add(Package="c", Version="2.0")
o.write_opk()
o.write_list()

opkgcl.update()
opkgcl.upgrade("--autoremove")

if not opkgcl.is_installed("b", "1.0"):
    opk.fail("b has been removed even though a still depends on it")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Create package 'a' (1.0) which Recommends 'c'.
# Install 'a' with --add-ignore-recommends 'c'.
# Check that only 'a' (1.0) is installed.
# Create package 'b' which Depends on 'c'.
# Install 'a' & 'b', with --add-ignore-recommends 'c'.
# Verify that 'a','b' & 'c' are installed.
# Uninstall 'b' & 'c'.
# Create package 'a' (2.0), which Recommends 'c'.
# Upgrade 'a' with --add-ignore-recommends 'c'
# Verify that only 'a' (2.0) is installed
#

import os
import opk, cfg, opkgcl

opk.regress_init()
o = opk.OpkGroup()

o.add(Package='a', Recommends='c, d', Version='1.0')
o.add(Package='b', Depends='c')
o.add(Package='c')
o.add(Package='d')
o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.install('a', '--add-ignore-recommends c')

if not opkgcl.is_installed('a'):
    opk.fail("Package 'a' installed but reports as not installed.")

if opkgcl.is_installed('c'):
    opk.xfail("[libsolv<0.7.3] Package 'c' should not have been installed since it was in --add-ignore-recommends.")

if not opkgcl.is_installed('d'):
    opk.fail("Package 'd' installed but reports as not installed.")

opkgcl.remove('a')
opkgcl.remove('d')
opkgcl.install('a b', '--add-ignore-recommends c')

if not opkgcl.is_installed('a'):
    opk.fail("Package 'a' installed but reports as not installed.")

if not opkgcl.is_installed('b'):
    opk.fail("Package 'b' installed but reports as not installed.")

if not opkgcl.is_installed('c'):
    opk.fail("Package 'c' should have been installed since 'b' depends on it.")

if not opkgcl.is_installed('d'):
    opk.fail("Package 'd' installed but reports as not installed.")

opkgcl.remove('b c d', '--force-depends')
o.add(Package='a', Recommends='c', Version='2.0')
o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.upgrade('a', "--add-ignore-recommends '*'")

if not opkgcl.is_installed('a', '2.0'):
    opk.fail("Package 'a (2.0)' installed but reports as not installed.")

if opkgcl.is_installed('c'):
    opk.fail("Package 'c' should not have been installed since it was in --add-ignore-recommends.")

if opkgcl.is_installed('d'):
    opk.fail("Package 'd' should not have been installed since it was in --add-ignore-recommends.")



#!/usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()

# Package A creates a symlink
os.mkdir('lib64')
os.symlink('lib64', 'lib')
with open('static_file.txt', 'w') as f:
    f.write('Test')
pkg = opk.Opk(Package='a')
pkg.write(data_files=['lib', 'lib64', 'static_file.txt'])
o.addOpk(pkg)
os.remove('static_file.txt')

# Package B creates the same symlink
pkg = opk.Opk(Package='b')
pkg.write(data_files=['lib', 'lib64'])
o.addOpk(pkg)
os.remove('lib')
os.rmdir('lib64')

# Package C creates a different symlink
os.mkdir('lib32')
os.symlink('lib32', 'lib')
pkg = opk.Opk(Package='c')
pkg.write(data_files=['lib', 'lib32'])
o.addOpk(pkg)
os.remove('lib')
os.rmdir('lib32')

# Package D creates a symlink to a file (but doesn't include the file)
os.symlink('static_file.txt', 'link.txt')
pkg = opk.Opk(Package='d')
pkg.write(data_files=['link.txt'])
o.addOpk(pkg)

# Package E creates the same file symlink
pkg = opk.Opk(Package='e')
pkg.write(data_files=['link.txt'])
o.addOpk(pkg)
os.remove('link.txt')

o.write_list()

opkgcl.update()

opkgcl.install('a')
opkgcl.install('b')
if not opkgcl.is_installed('b'):
    opk.fail("Package 'b' could not be installed")

opkgcl.install('c')
if opkgcl.is_installed('c'):
    opk.fail("Package 'c' installed despite symlink conflict")

opkgcl.install('d')
opkgcl.install('e')
if opkgcl.is_installed('e'):
    opk.fail("Package 'e' installed despite symlink conflict")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Reporter: alejandro.delcastillo@ni.com
#
# What steps will reproduce the problem?
# ======================================
#
# 1.- Create package a (v 1.0) that Provides b and c, Replaces b, Conflicts with b.
#         install it
# 2.- Create package a (v 2.0) that Provides b and c, Replaces b, Conflicts with b.
#         upgrade
#
# What is the expected output? What do you see instead?
# =====================================================
#
# Upgrade fails
#

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a", Version="1.0", Provides="b, c", Replaces="b", Conflicts="b")
o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.install("a")

o = opk.OpkGroup()
o.add(Package="a", Version="2.0", Provides="b, c", Replaces="b", Conflicts="b")
o.write_opk()
o.write_list()

opkgcl.update()
status = opkgcl.upgrade()

if not opkgcl.is_installed("a", "2.0"):
    opk.fail("New version of package 'a' available during upgrade but was not installed")

opkgcl.remove("a")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Reporter: Eric Yu
#
# What steps will reproduce the problem?
# ======================================
#
# 1. Create packages 'a' and 'b', where 'a' depends on 'b'.
# 2. Install 'a' (which also installs 'b')
# 3. Remove 'b' with --force-removal-of-dependent-packages
#
# What is the expected output? What do you see instead?
# =====================================================
#
# Removing 'b' with --force-removal-of-dependent-packages should remove
# both 'b' and 'a'.
#
# Instead, only 'b' is removed, leaving 'a' installed.
#
# Status
# ======
# Open
#

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a", Depends="b")
o.add(Package="b")

o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.install("a")
if not opkgcl.is_installed("a"):
        opk.fail("Package 'a' installed but reports as not installed")
if not opkgcl.is_installed("b"):
        opk.fail("Package 'b' should be installed as a dependency of 'a' "
                 "but reports as not installed")

opkgcl.remove("b","--force-removal-of-dependent-packages")
if opkgcl.is_installed("b"):
        opk.fail("Package 'b' removed but reports as installed.")
if opkgcl.is_installed("a"):
        opk.fail("Package 'b' removed with --force-removal-of-dependent-packages "
                  "but 'a' which depends on 'b' still installed.")



#!/usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Reporter: Rasmus Villemoes <rasmus.villemoes@prevas.dk>
#
# What steps will reproduce the problem?
########################################
#
# 1. pkg 'a' recommends pkg 'dne'
# 2. pkg 'dne' does not exist in the feed
# 3. ask opkg to `install a dne`
#
# What is the expected output? What do you see instead?
########################################
# Expected:
#  The opkg install operation should fail, and opkg should report that it cannot
#  find a package to satisfy 'dne'.
# Actual:
#  The opkg install operation succeeds, with no indication that package 'dne' is
#  not satisfiable.

import os
import opk, cfg, opkgcl

opk.regress_init()
o = opk.OpkGroup()

o.add(Package='a', Recommends='dne')
o.write_opk()
o.write_list()

opkgcl.update()

rc, stdout = opkgcl.opkgcl('install a dne')

assert rc != 0, \
    "Opkg install returned code 0; but package 'dne' should not be satisfiable."



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Install a package 'a' at version 1.0. Make available 'a' at version 2.0 with
# dependency on 'b', and make available 'b'. Do upgrade and ensure that 'b' in
# installed correctly.
#

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a", Version="1.0")
o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.install("a")
if not opkgcl.is_installed("a"):
    opk.fail("Package 'a' installed but reports as not installed.")

o = opk.OpkGroup()
o.add(Package="a", Version="2.0", Depends="b")
o.add(Package="b")
o.write_opk()
o.write_list()

opkgcl.update()
opkgcl.upgrade()

if not opkgcl.is_installed("a", "2.0"):
    opk.fail("New version of package 'a' available during upgrade but was not installed")
if opkgcl.is_installed("a", "1.0"):
    opk.fail("Package 'a' upgraded but old version still installed.")
if not opkgcl.is_installed("b"):
    opk.fail("Package 'b' should be installed as a dependency of 'a' but does not report as installed.")

# Check the packages are marked correctly
if opkgcl.is_autoinstalled("a"):
    opk.fail("Package 'a' explicitly installed by reports as auto installed.")
if not opkgcl.is_autoinstalled("b"):
    opk.fail("Package 'b' installed as a dependency but does not report as auto installed.")



# SPDX-License-Identifier: GPL-2.0-only
import errno
import hashlib
import os
import os.path
import stat
import sys
import tarfile
from pathlib import Path

import cfg

__appname = sys.argv[0]


class Opk:

    valid_control_fields = [
        'Architecture',
        'Conffiles',
        'Conflicts',
        'Depends',
        'Description',
        'Essential',
        'Filename',
        'Homepage',
        'InstalledSize',
        'MD5Sum',
        'Maintainer',
        'OE',
        'Package',
        'Priority',
        'Provides',
        'Recommends',
        'Replaces',
        'Section',
        'Size',
        'Source',
        'Suggests',
        'Version',
        ]

    control = None
    postinst = None
    postrm = None
    preinst = None
    prerm = None

    def __init__(self, subdirectory=None, **control):
        for k in control.keys():
            if k not in self.valid_control_fields:
                raise Exception('Invalid control field: {}'.format(k))
        if 'Package' not in control.keys():
            print('Cannot create opk without Package name.\n')
            return None
        if 'Architecture' not in control.keys():
            control['Architecture'] = 'all'
        if 'Version' not in control.keys():
            control['Version'] = '1.0'
        if subdirectory is not None:
            self._relative_dir = Path(subdirectory)
        else:
            self._relative_dir = None
        self.control = control

    def write(self, tar_not_ar=False, data_files=None, compression='gz'):
        COMPRESSORS = ['gz', 'bz2', 'xz']
        if compression not in COMPRESSORS:
            raise Exception('Invalid compression type: {}. '
                            'Supported compressors: {}'.format(
                                compression,
                                ', '.join(COMPRESSORS)))

        control_file = 'control.tar.' + compression
        data_file = 'data.tar.' + compression
        tar_mode = 'w:' + compression

        TEMP_FILES = ['control', control_file, data_file, 'preinst',
                      'postinst', 'prerm', 'postrm', 'debian-binary']

        # process a final filename for the package
        dirname = self._relative_dir or ''
        basename = self.control.get(
            'Filename',
            '{Package}_{Version}_{Architecture}.opk'.format(**self.control)
            )
        filename = os.path.join(dirname, basename)
        self.control['Filename'] = filename

        for f in TEMP_FILES + [filename]:
            if os.path.exists(f):
                os.unlink(f)

        with open('debian-binary', 'w') as f:
            f.write('2.0\n')

        with open('control', 'w') as f:
            for k in self.control.keys():
                f.write('{}: {}\n'.format(k, self.control[k]))

        if self.preinst:
            with open('preinst', 'w') as f:
                os.fchmod(f.fileno(), 0o755)
                f.write(self.preinst)

        if self.postinst:
            with open('postinst', 'w') as f:
                os.fchmod(f.fileno(), 0o755)
                f.write(self.postinst)

        if self.prerm:
            with open('prerm', 'w') as f:
                os.fchmod(f.fileno(), 0o755)
                f.write(self.prerm)

        if self.postrm:
            with open('postrm', 'w') as f:
                os.fchmod(f.fileno(), 0o755)
                f.write(self.postrm)

        with tarfile.open(control_file, tar_mode) as tar:
            tar.add('control')
            if self.preinst: tar.add('preinst')
            if self.postinst: tar.add('postinst')
            if self.prerm: tar.add('prerm')
            if self.postrm: tar.add('postrm')

        with tarfile.open(data_file, tar_mode) as tar:
            if data_files:
                for df in data_files:
                    tar.add(df)

        if self._relative_dir is not None:
            self._relative_dir.mkdir(parents=True, exist_ok=True)
        if tar_not_ar:
            with tarfile.open(filename, tar_mode) as tar:
                tar.add(control_file)
                tar.add(data_file)
        else:
            os.system('ar q {0} {1} {2} {3} 2>/dev/null'.format(
                filename,
                'debian-binary',
                control_file,
                data_file))

        for f in TEMP_FILES:
            if os.path.exists(f):
                os.unlink(f)


def md5sum_file(fname):
    f = open(fname, 'rb')
    md5 = hashlib.md5()
    chunk_size = 4096
    for chunk in iter(lambda: f.read(chunk_size), b''):
        md5.update(chunk)
    return md5.hexdigest()


class OpkGroup:
    def __init__(self):
        self.opk_list = []

    def add(self, **control):
        opk = Opk(**control)
        self.opk_list.append(opk)
        return opk

    def addOpk(self, opk):
        self.opk_list.append(opk)

    def removeOpk(self, opk):
        self.opk_list.remove(opk)

    def write_list(self, filename='Packages'):
        f = open(filename, 'w')
        for opk in self.opk_list:
            for k in opk.control.keys():
                f.write('{}: {}\n'.format(k, opk.control[k]))
            fname = opk.control['Filename']
            fsize = os.stat(fname).st_size
            md5sum = md5sum_file(fname)
            f.write('Size: {}\n'.format(fsize))
            f.write('MD5Sum: {}\n'.format(md5sum))
            f.write('\n')
        f.close()

    def write_opk(self, tar_not_ar=False, compression='gz'):
        for o in self.opk_list:
            o.write(tar_not_ar=tar_not_ar, compression=compression)


def fail(msg):
    print('%s: Test failed: %s' % (__appname, msg))
    exit(-1)


# Expected failure: For issues we know haven't been fixed yet. As these aren't
# regressions don't return an error condition.
def xfail(msg):
    print('%s: Expected failure: %s' % (__appname, msg))
    exit(0)


def regress_init():
    """Initialisation and sanity checking."""

    if not os.access(cfg.opkgcl, os.X_OK):
        fail('Cannot exec {}'.format(cfg.opkgcl))

    try:
        os.makedirs(cfg.opkdir)
    except OSError as exception:
        if exception.errno != errno.EEXIST:
            raise

    os.chdir(cfg.opkdir)

    os.system('rm -fr {}'.format(cfg.offline_root))

    confdir = os.environ['SYSCONFDIR'] + '/opkg'
    os.makedirs(('{}'+confdir).format(cfg.offline_root))
    f = open(('{}'+confdir+'/opkg.conf').format(cfg.offline_root), 'w')
    f.write('arch all 1\n')
    f.write('src test file:{}\n'.format(cfg.opkdir))
    f.close()



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Reporter: Eric Yu
#
# What steps will reproduce the problem?
# ======================================
#
# 1. Create packages 'x', 'a','b', and 'c' such that 'a' recommends 'b' and 'c',
#    and 'c' conflicts 'x'.
# 2. Install 'x'.
# 3. Try to install 'a'.
#
# What is the expected output? What do you see instead?
# =====================================================
#
# 'a' and 'b' should be installed, and 'c' should remain uninstalled.
#
# Instead, only 'b' is installed and 'a' remains uninstalled.
#
# Status
# ======
# Open
#

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()

o.add(Package="x")
o.add(Package="a", Recommends="b, c")
o.add(Package="b")
o.add(Package="c", Conflicts="x")

o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.install("x");
if not opkgcl.is_installed("x"):
    opk.fail("Package 'x' installed but reports as not installed")

opkgcl.install("a");
if not opkgcl.is_installed("x"):
    opk.fail("Package 'x' removed when 'a' installed, but should not be removed.")
if not opkgcl.is_installed("b"):
    opk.xfail("[internalsolv] Package 'b' not installed, but should be installed as a recommended pkg of 'a'.")
if opkgcl.is_installed("c"):
    opk.fail("Package 'c' installed but conflicts with x.")
if not opkgcl.is_installed("a"):
    opk.fail("Package 'a' not installed even though 'c' is not an absolute dependency of 'a'.")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Create package 'x(1.0)' with architecture a and 'x(1.0)' with architecture b.
# Install 'x' with --add-arch a:2 and --add-arch b:3.
# Check that 'x(1.0), arch b' is installed.
#

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="x", Version="1.0", Architecture="b")
o.add(Package="x", Version="1.0", Architecture="a")

o.write_opk()
o.write_list()

opkgcl.update()

# install should pick arch=b version
opkgcl.install("x", "--add-arch a:2 --add-arch b:3")
if opkgcl.is_installed("x", "1.0", "--add-arch a:2"):
        opk.fail("Package 'x(1.0), arch b' available but arch a installed")
if not opkgcl.is_installed("x", "1.0", "--add-arch b:3"):
        opk.fail("Package 'x(1.0), arch=b' available but was not installed.")

opkgcl.install("x", "--force-reinstall --add-arch a:2 --add-arch b:3")
if not opkgcl.is_installed("x", "1.0", "--add-arch b:3"):
        opk.fail("Package 'x(1.0), arch=b' available but was not re-installed.")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Install a package and mark it as on hold. Make an updated version available
# and upgrade. Check that the new version is not installed and the old version
# remains installed.
#

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a", Version="1.0")
o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.install("a")
if not opkgcl.is_installed("a"):
    opk.fail("Package 'a' installed but reports as not installed.")

opkgcl.flag_hold("a")

# Make a new version of 'a' available
o.add(Package="a", Version="2.0")
o.write_opk()
o.write_list()

opkgcl.update()
opkgcl.upgrade()

if opkgcl.is_installed("a", "2.0"):
    opk.fail("Old version of package 'a' flagged as on hold but was upgraded.")
if not opkgcl.is_installed("a", "1.0"):
    opk.fail("Package 'a' not upgraded but old version was removed.")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Create packages 'a' and 'b' which both depend on 'c'.
# Install 'a' and 'b' and check that 'c' is autoinstalled.
# Remove 'a' with --autoremove and check that 'c' is not autoremoved.
# Remove 'b' with --autoremove and check that 'c' is autoremoved.
#

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a", Depends="c")
o.add(Package="b", Depends="c")
o.add(Package="c")

o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.install("a")
if not opkgcl.is_installed("a"):
        opk.fail("Package 'a' installed but does not report as installed.")
if not opkgcl.is_installed("c"):
        opk.fail("Package 'c' should be installed as a dependency of 'a' but does not report as installed.")

opkgcl.install("b")
if not opkgcl.is_installed("b"):
    opk.fail("Package 'b' installed but does not report as installed.")

# Check the packages are marked correctly
if opkgcl.is_autoinstalled("a"):
    opk.fail("Package 'a' explicitly installed by user but reports as auto installed.")
if opkgcl.is_autoinstalled("b"):
        opk.fail("Package 'b' explicitly installed by user but reports as auto installed.")
if not opkgcl.is_autoinstalled("c"):
    opk.fail("Package 'c' installed as a dependency but does not report as auto installed.")

# Check that autoinstalled packages are not removed when other packages still depend on them
opkgcl.remove("a","--autoremove")
if opkgcl.is_installed("a"):
        opk.fail("Package 'a' removed but reports as installed.")
if not opkgcl.is_installed("c"):
        opk.fail("Package 'c' depended upon by 'b' but was autoremoved.")

# Check that autoinstalled packages are removed when no other packages depend on them
opkgcl.remove("b","--autoremove")
if opkgcl.is_installed("b"):
        opk.fail("Package 'b' removed but reports as installed.")
if opkgcl.is_installed("c"):
        opk.fail("Package 'c' not removed from --autoremove.")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Reporter: michal.p...@gmail.com
#
# What steps will reproduce the problem?
# ======================================
#
# 1. Have package A (1.0)  depending on package B (1.0).
# 2. On the repo, add package A (2.0) and pacakge B (2.0)
# 3. Do 'opkg-cl upgrade'
#
#
# What is the expected output? What do you see instead?
# =====================================================
#
# opkg should upgrade package A and B to version 2 and exit with 0.
# It correctly updates the packages, but exits with -1.
#
# Status: Resolved
#

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a", Version="1.0", Depends="b")
o.add(Package="b", Version="1.0")
o.write_opk()
o.write_list()

opkgcl.update()
opkgcl.install("a")

o = opk.OpkGroup()
o.add(Package="a", Version="2.0", Depends="b")
o.add(Package="b", Version="2.0")
o.write_opk()
o.write_list()

opkgcl.update()
status = opkgcl.upgrade()
if not opkgcl.is_installed("a", "2.0"):
    opk.fail("New version of package 'a' available during upgrade but was not installed")
if not opkgcl.is_installed("b", "2.0"):
    opk.fail("New version of package 'b' available during upgrade but was not installed")
if status != 0:
    opk.fail("Return value was different than 0")
opkgcl.remove("a")
opkgcl.remove("b")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only

import os
import opk, cfg, opkgcl

opk.regress_init()

_, version = opkgcl.opkgcl("--version")
libsolv = True if 'libsolv' in version else False

open("asdf", "w").close()
a = opk.Opk(Package="a", Version="1.0", Depends="b")
a.write()
b = opk.Opk(Package="b", Version="1.0")
b.write(data_files=["asdf"])

o = opk.OpkGroup()
o.addOpk(a)
o.addOpk(b)
o.write_list()

opkgcl.update()
opkgcl.install("a")

if not opkgcl.is_autoinstalled("b"):
    opk.fail("b is not autoinstalled")

a = opk.Opk(Package="a", Version="2.0", Depends="b")
a.write()
b = opk.Opk(Package="b", Version="2.0")
b.write(data_files=["asdf"])

o = opk.OpkGroup()
o.addOpk(a)
o.addOpk(b)
o.write_list()

opkgcl.update()
opkgcl.upgrade();

if not opkgcl.is_autoinstalled("b"):
    opk.fail("b is not autoinstalled anymore")

# The internalsolver is more agressive removing orphans than libsolv. On libsolv upgrades, no
# packages are removed (on dist-upgrade, removal is allowed)
if not libsolv:
    a = opk.Opk(Package="a", Version="3.0")
    a.write(data_files=["asdf"])
    os.unlink("asdf")

    o = opk.OpkGroup()
    o.addOpk(a)
    o.write_list()

    opkgcl.update()
    opkgcl.upgrade();

    if opkgcl.is_installed("b", "2.0"):
        opk.fail("b is still installed")

    if not opkgcl.is_installed("a", "3.0"):
        opk.fail("a is not installed")



#!/usr/bin/python3
# SPDX-License-Identifier: GPL-2.0-only
import os
import subprocess

import cfg

debug_opkg_cmds = bool(os.environ.get('DEBUG_OPKG_CMDS') or False)
vardir = os.environ['VARDIR']


def opkgcl(opkg_args):
    cmd = '{} -o {} {}'.format(cfg.opkgcl, cfg.offline_root, opkg_args)
    if debug_opkg_cmds:
        print('DEBUG cmd: {}'.format(cmd))
    p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE,
                         stderr=subprocess.STDOUT)
    (stdout_data, stderr_data) = p.communicate()
    if debug_opkg_cmds:
        print('DEBUG stdout: {}'.format(stdout_data.decode('utf-8') if stdout_data else None))
        print('DEBUG stderr: {}'.format(stderr_data.decode('utf-8') if stderr_data else None))
    status = p.returncode
    return (status, stdout_data.decode('utf-8'))


def distupgrade(params=None, flags=''):
    if params:
        return opkgcl('{} --force-postinstall dist-upgrade {}'.format(flags, params))[0]
    else:
        return opkgcl('--force-postinstall dist-upgrade')[0]


def download(pkg_name, flags=''):
    return opkgcl('{} download {}'.format(flags, pkg_name))[0]


def files(pkg_name):
    output = opkgcl('files {}'.format(pkg_name))[1]
    return output.split('\n')[1:]


def flag_hold(pkg_name):
    out = opkgcl('flag hold {}'.format(pkg_name))
    return out == 'Setting hold flag on package {}.'.format(pkg_name)


def flag_unpacked(pkg_name):
    out = opkgcl('flag unpacked {}'.format(pkg_name))
    return out == 'Setting flags for package {} to unpacked.'.format(pkg_name)


def info(pkg_name, params=None, flags=''):
    if params:
        return opkgcl('{} --fields {} info {}'.format(flags, params, pkg_name))[1]
    else:
        return opkgcl('{} info {}'.format(flags, pkg_name))[1]


def install(pkg_name, flags=''):
    return opkgcl('{} --force-postinstall install {}'.format(flags, pkg_name))[0]


def is_autoinstalled(pkg_name):
    status_path = ('{}' + vardir + '/lib/opkg/status').format(cfg.offline_root)
    if not os.path.exists(status_path):
        return False
    status_file = open(status_path, 'r')
    status = status_file.read()
    status_file.close()
    index_start = status.find('Package: {}'.format(pkg_name))
    if index_start < 0:
        return False
    index_end = status.find('\n\n', index_start)
    return status.find('Auto-Installed: yes', index_start, index_end) >= 0


def is_installed(pkg_name, version=None, flags=''):
    out = opkgcl('{} list_installed {}'.format(flags, pkg_name))[1]
    if len(out) == 0 or out.split()[0] != pkg_name:
        return False
    if version and out.split()[2] != version:
        return False
    if not os.path.exists(('{}' + vardir + '/lib/opkg/info/{}.control')\
                .format(cfg.offline_root, pkg_name)):
        return False
    return True


def is_upgradable(pkg_name, version=None):
    out = opkgcl('list_upgradable')[1]

    if len(out) == 0:
        return False

    for line in out.splitlines():
        pkg, *_, new_v = line.split()

        if pkg != pkg_name:
            continue

        if version and new_v != version:
            return False

        return True

    return False


def remove(pkg_name, flags=''):
    return opkgcl('{} --force-postinstall remove {}'.format(flags, pkg_name))[0]


def update():
    return opkgcl('update')[0]


def search(glob):
    (status, output) = opkgcl('search {}'.format(glob))
    if status == 255:
        raise FileNotFoundError()
    elif status != 0:
        raise OSError(status)
    else:
        return output.strip().split('\n')


def upgrade(params=None, flags=''):
    if params:
        return opkgcl('{} --force-postinstall upgrade {}'.format(flags, params))[0]
    else:
        return opkgcl('--force-postinstall upgrade')[0]


if __name__ == '__main__':
    import sys
    (status, output) = opkgcl(' '.join(sys.argv[1:]))
    print(output)
    exit(status)



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Install a package A at version V. Install a package B, which conflicts
# with a version of A older than V. Make available A at a version newer
# than V. Do upgrade and ensure that A gets installed correctly.
#

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a", Version="2.0")
o.add(Package="b", Version="1.0", Conflicts="a (<< 1.0)")
o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.install("a")
opkgcl.install("b")

o = opk.OpkGroup()
o.add(Package="a", Version="3.0")
o.write_opk()
o.write_list()

opkgcl.update()
opkgcl.upgrade()

if not opkgcl.is_installed("a", "3.0"):
    opk.fail("New version of package 'a' available during upgrade but was not installed")



#!/usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only

import os
import opk, cfg, opkgcl
import re
vardir=os.environ['VARDIR']

re_half_installed = re.compile('Status: \w+ \w+ half-installed')

def is_half_installed(pkg_name):
    status_path = ("{}"+vardir+"/lib/opkg/status").format(cfg.offline_root)
    if not os.path.exists(status_path):
        return False
    with open(status_path, "r") as status_file:
        status = status_file.read()
    index_start = status.find("Package: {}".format(pkg_name))
    while index_start >= 0:
        index_end = status.find("\n\n", index_start)
        match = re_half_installed.search(status[index_start:index_end])
        if match is not None:
            return True
        index_start = status.find("Package: {}".format(pkg_name), index_end)
    return False

opk.regress_init()

o = opk.OpkGroup()

with open('conflict_file', 'w') as f:
    f.write('Test')
a = opk.Opk(Package='a', Version='1.0')
a.write(data_files=['conflict_file'])
b = opk.Opk(Package='b', Version='1.0')
b.write(data_files=['conflict_file'])

o.opk_list.append(a)
o.opk_list.append(b)
o.write_list()

opkgcl.update()

opkgcl.install('a')
opkgcl.install('b')
if not opkgcl.is_installed('a', '1.0'):
    opk.fail("Package 'a' failed to install")
if opkgcl.is_installed('b'):
    opk.fail("Package 'b' installed despite file conflict")

opkgcl.remove('b')
if opkgcl.is_installed('b'):
    opk.fail("Package 'b' failed to remove")
if is_half_installed('b'):
    opk.fail("Failed to clean up package 'b' from status file")

b2 = opk.Opk(Package='b', Version='2.0')
b2.write()

o.opk_list.append(b2)
o.write_list()

opkgcl.update()

opkgcl.install('b')
if not opkgcl.is_installed('b', '2.0'):
    opk.fail("Package 'b' failed to install")
if is_half_installed('b'):
    opk.fail("Failed to clean up previous package 'b' from status file")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a", Depends="b")
o.add(Package="b", Version="1.0")
o.add(Package="b", Version="2.0")
o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.install("b_1.0_all.opk")
if not opkgcl.is_installed("b", "1.0"):
    opk.fail("Package 'b' failed to install")

opkgcl.install("a")
if not opkgcl.is_installed("a"):
    opk.fail("Package 'a' failed to install")
if not opkgcl.is_installed("b", "1.0"):
    opk.fail("Package 'b' upgraded but upgrade was not necessary")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()

o.add(Package='a', Recommends='b, c (> 1.0), d')
o.add(Package='b')
o.add(Package='c', Version='2.0' )
o.add(Package='d')
o.add(Package='e')
o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.install('a', '--add-ignore-recommends b')

if not opkgcl.is_installed('a'):
    opk.fail("Package 'a' installed but reports as not installed.")

if not opkgcl.is_installed('c'):
    opk.fail("Package 'c' installed but reports as not installed.")

if not opkgcl.is_installed('d'):
    opk.fail("Package 'd' installed but reports as not installed.")

if opkgcl.is_installed('b'):
    opk.xfail("[libsolv<0.7.3] Package 'b' should not have been installed since it was in --add-ignore-recommends.")

opkgcl.install('e')

if not opkgcl.is_installed('e'):
    opk.fail("Package 'e' installed but reports as not installed.")
if opkgcl.is_installed('b'):
    opk.fail("Package 'b' should not have been installed since 'c' doesn't depends/recommends it.")

opkgcl.remove("'*'", "--force-depends")

opkgcl.install('a', '--add-ignore-recommends c')

if not opkgcl.is_installed('a'):
    opk.fail("Package 'a' installed but reports as not installed.")

if not opkgcl.is_installed('b'):
    opk.fail("Package 'b' installed but reports as not installed.")

if not opkgcl.is_installed('d'):
    opk.fail("Package 'd' installed but reports as not installed.")

if opkgcl.is_installed('c'):
    opk.xfail("Package 'c' should not have been installed since it was in --add-ignore-recommends.")

opkgcl.install('e')

if not opkgcl.is_installed('e'):
    opk.fail("Package 'e' installed but reports as not installed.")
if opkgcl.is_installed('c'):
    opk.fail("Package 'c' should not have been installed since 'c' doesn't depends/recommends it.")


#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Issue 124, reported by muusclaus
#
# Install package 'a' version 1.0 which depends on package 'b' version 1.0. Then
# make available 'a' version 1.1 which depends on 'b' version 1.1. Attempt to
# upgrade 'b' to version 1.1.
#
# The upgrade should be blocked as the installed version of 'a' (1.0) depends
# explictly on 'b' version 1.0 and this dependency is not satisfied by 'b'
# version 1.1.

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a", Version="1.0", Depends="b (= 1.0)")
o.add(Package="b", Version="1.0")
o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.install("a")
if not opkgcl.is_installed("a"):
    opk.fail("Package 'a' failed to install.")
if not opkgcl.is_installed("b"):
    opk.fail("Package 'b' not installed despite dependency from package 'a'.")

o = opk.OpkGroup()
o.add(Package="a", Version="1.1", Depends="b (= 1.1)")
o.add(Package="b", Version="1.1")
o.write_opk()
o.write_list()

opkgcl.update()

# 'opkg upgrade b' should fail as it won't upgrade a
opkgcl.upgrade("b")

# Check 'a' has not been upgraded
if opkgcl.is_installed("a", "1.1"):
    opk.xfail("[libsolv] Package 'a' upgraded despite not being listed in packages to upgrade.")
if not opkgcl.is_installed("a", "1.0"):
    opk.fail("Package 'a' version 1.0 removed.")

# Check 'b' has not been upgraded
if opkgcl.is_installed("b", "1.1"):
    opk.fail("Package 'b' upgraded despite breaking dependency of package 'a'.")
if not opkgcl.is_installed("b", "1.0"):
    opk.fail("Package 'b' version 1.0 removed.")

# 'opkg upgrade a' should succeed and upgrade both a and b
opkgcl.upgrade("a")

# Check 'a' has been upgraded
if not opkgcl.is_installed("a", "1.1"):
    opk.fail("Package 'a' failed to upgrade.")
if opkgcl.is_installed("a", "1.0"):
    opk.fail("Package 'a' version 1.0 not removed despite successful upgrade.")

# Check 'b' has been upgraded
if not opkgcl.is_installed("b", "1.1"):
    opk.fail("Package 'b' failed to upgrade.")
if opkgcl.is_installed("b", "1.0"):
    opk.fail("Package 'b' version 1.0 not removed despite successful upgrade.")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Reporter: Damine Gotfroi <gotfroi.damien@gmail.com>
#
# What steps will reproduce the problem?
########################################
#
# 1. Create package 'a' (1.0), which DEPENDS on 'b'.
# 2. Create package 'b' (1.0), which DEPENDS on 'c' and Provides 'd'
# 3. Create package 'c' (1.0), which PROVIDES 'e'
# 4. Install 'a'
# 5. Create package 'a' (2.0), which DEPENDS on 'f'.
# 6. Create package 'f' (2.0), which DEPENDS on 'g' and Provides 'd'
# 7. Create package 'g' (2.0), which PROVIDES 'e'
# 8. Upgrade
#
# What is the expected output? What do you see instead?
########################################
# Packages 'a' (2.0), 'b' (1.0),  'c' (1.0), f (2.0) and g (2.0) should be installed.
#
# Instead, opkg only packages 'a' (2.0) and c (1.0) are installed
#

import os
import opk, cfg, opkgcl

opk.regress_init()

_, version = opkgcl.opkgcl("--version")
libsolv = True if 'libsolv' in version else False

o = opk.OpkGroup()
o.add(Package='a', Version='1.0', Depends='b')
o.add(Package='b', Version='1.0', Depends='c', Provides='d')
o.add(Package='c', Version='1.0', Provides='e')
o.write_opk()
o.write_list()
opkgcl.update()

opkgcl.install('a')

if not opkgcl.is_installed('a', '1.0'):
    opk.fail("Package 'a' (1.0)  installed but does not report as installed.")
if not opkgcl.is_installed('b', '1.0'):
    opk.fail("Package 'b (1.0)' should be installed as a dependency of 'a' but does not report as installed.")
if not opkgcl.is_installed('c', '1.0'):
    opk.fail("Package 'c' (1.0) or 'e' should be installed as a dependency of 'b (1.0)' but does not report as installed.")

o.add(Package='a', Version='2.0', Depends='f')
o.add(Package='f', Version='2.0', Depends='g', Provides='d')
o.add(Package='g', Version='2.0', Provides='e')
o.write_opk()
o.write_list()
opkgcl.update()
opkgcl.upgrade()

if not opkgcl.is_installed('a', '2.0'):
    opk.fail("Package 'a' (2.0)  installed but does not report as installed.")
# The internalsolver is more agressive removing orphans than libsolv. For the moment, that's by design and sounds
# dangerous to modify the behavior
if not opkgcl.is_installed('b', '1.0') and libsolv:
    opk.fail("Package 'b (1.0)' should be installed as a dependency of 'a' but does not report as installed.")
if not opkgcl.is_installed('c', '1.0'):
    opk.fail("Package 'c' (1.0) should be installed as a dependency of 'b (1.0)' but does not report as installed.")
if not opkgcl.is_installed('f', '2.0'):
    opk.fail("Package 'f' (2.0) should be installed as a dependency of 'a (2.0)' but does not report as installed.")
if not opkgcl.is_installed('g', '2.0'):
    opk.fail("Package 'g' (2.0) should be installed as a dependency of 'f (2.0)' but does not report as installed.")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Create essential packages 'a', which Depends on b. Package 'b' depends on 'c'
# Install 'a'.
# Remove 'c' using --force-removal-of-dependent-packages
# Make sure nothing is removed.

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package='a', Depends='b', Essential='yes')
o.add(Package='b', Depends='c')
o.add(Package='c')

o.write_opk()
o.write_list()
opkgcl.update()

opkgcl.install('a')
if not opkgcl.is_installed('a'):
        opk.fail("Package 'a' installed but does not report as installed.")
if not opkgcl.is_installed('b'):
        opk.fail("Package 'b' installed but does not report as installed.")
if not opkgcl.is_installed('c'):
        opk.fail("Package 'c' installed but does not report as installed.")

opkgcl.remove('c', '--force-removal-of-dependent-packages')
if not opkgcl.is_installed('b'):
        opk.xfail("[internalsolv] Package 'b' was removed, but Essential package 'a' depends on it")
if not opkgcl.is_installed('c'):
        opk.fail("Package 'c' was removed, which breaks Essential package 'a'")
if not opkgcl.is_installed('a'):
        opk.fail("Essential package 'a' was incorrectly removed")



#!/usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()

# Package A creates a symlink
os.mkdir('ro')
os.symlink('../rw/config', 'ro/config')
pkg = opk.Opk(Package='a')
pkg.write(data_files=['ro'])
o.addOpk(pkg)
os.remove('ro/config')

# Package B puts files in the directory
os.mkdir('ro/config')
with open('ro/config/b.conf', 'w') as f:
    f.write('Test')
pkg = opk.Opk(Package='b')
pkg.write(data_files=['ro'])
o.addOpk(pkg)
os.remove('ro/config/b.conf')
os.rmdir('ro/config')
os.rmdir('ro')

o.write_list()

opkgcl.update()

opkgcl.install('a')
if not os.path.islink("%s/ro/config" % cfg.offline_root):
    opk.fail("Package 'a' didn't install required file")
opkgcl.install('b')
if opkgcl.is_installed('b'):
    opk.fail("Package 'b' installed despite symlink conflict")
opkgcl.remove('b')
opkgcl.remove('a')

opkgcl.install('b')
if not os.path.exists("%s/ro/config/b.conf" % cfg.offline_root):
    opk.fail("Package 'b' didn't install required file")
opkgcl.install('a')
if opkgcl.is_installed('a'):
    opk.fail("Package 'a' installed despite symlink conflict")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()

# Package 'a' creates a dir, and a symlink to that dir
os.mkdir('lib64')
os.symlink('lib64', 'lib')
open('lib64/testfile.txt', 'w').close()
a = opk.Opk(Package='a')
a.write(data_files=['lib', 'lib64', 'lib64/testfile.txt'])
o.addOpk(a)
os.remove('lib64/testfile.txt')
os.remove('lib')
os.rmdir('lib64')

# Package 'b' creates a file in a dir that follows the symlink created by 'a'
os.mkdir('lib')
open('lib/testfile2.txt', 'w').close()
b = opk.Opk(Package='b')
b.write(data_files=['lib', 'lib/testfile2.txt'])
o.addOpk(b)
os.remove('lib/testfile2.txt')
os.rmdir('lib')

o.write_list()
opkgcl.update()

opkgcl.install('a')
if not os.path.exists("%s/lib64/testfile.txt" % cfg.offline_root):
    opk.fail("Package 'a' didn't install required file")

opkgcl.install('b')
if not opkgcl.is_installed('b'):
    opk.fail("Package 'b' could not be installed")

if not os.path.exists("%s/lib64/testfile2.txt" % cfg.offline_root):
    opk.fail("Package 'b' didn't install required file")

opkgcl.remove('a')
if not os.path.exists("%s/lib64/testfile2.txt" % cfg.offline_root):
    opk.fail("Package 'a' removed package 'b' file")

if not os.path.exists("%s/lib" % cfg.offline_root):
    opk.fail("Package 'a' incorrectly removed symlink that is in use by Package 'b'")

opkgcl.remove('b')
if os.path.exists("%s/lib64/testfile2.txt" % cfg.offline_root):
    opk.fail("Package 'b' removed but file 'testfile2.txt' still present.")



#!/usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()

with open('conflict_file', 'w') as f:
    f.write('Test')
a = opk.Opk(Package='a', Version='1.0')
a.write(data_files=['conflict_file'])
b = opk.Opk(Package='b', Version='1.0')
b.write(data_files=['conflict_file'])

o.opk_list.append(a)
o.opk_list.append(b)
o.write_list()

opkgcl.update()

opkgcl.install('a')
opkgcl.install('b')
if not opkgcl.is_installed('a', '1.0'):
    opk.fail("Package 'a' failed to install")
if opkgcl.is_installed('b'):
    opk.fail("Package 'b' installed despite file conflict")

b2 = opk.Opk(Package='b', Version='2.0')
b2.write(data_files=['conflict_file'])

o.opk_list.append(b2)
o.write_list()

opkgcl.update()

# Installing another version to ensure the state of the first gets updated
# even though the second also contains a conflict
opkgcl.install('b_2.0_all.opk')
if opkgcl.is_installed('b'):
    opk.fail("Package 'b' installed despite file conflict")

b3 = opk.Opk(Package='b', Version='3.0')
b3.write()

o.opk_list.append(b3)
o.write_list()

opkgcl.update()

# This should install a working version, removing the debris from the failed
# install attempts above
opkgcl.install('b_3.0_all.opk')
if not opkgcl.is_installed('b', '3.0'):
    opk.fail("Package 'b' failed to install")
